/************************************************************************
*  This file is part of the KDE project, module kcontrol.               *
*  Copyright (C) 2007 Ian Ventura-Whiting <fizz@titania.co.uk>          *
*                                                                       *
*  You can Freely distribute this program under the GNU LGPL version 2  *
*************************************************************************/

// Qt Includes...
#include <QGridLayout>
#include <QTreeWidget>
#include <QStringList>
#include <QSpacerItem>
#include <QDir>

// KDE Includes...
#include <KDialog>
#include <KAboutData>
#include <kdeversion.h>
#include <KIcon>
#include <KPushButton>
#include <KMessageBox>
#include <kgenericfactory.h>

// ODBC/SQL/DB Includes...
#include <odbcinstext.h>
#include <sqlext.h>
#include <sqltypes.h>

// Other Includes...
#include <unistd.h>

// Data Source Includes...
#include "datasources.h"
#include "dsn_dialog.h"

// Defines...
#define ODBC_BUFFER_SIZE 9600
#define ODBC_FILEDSN_SIZE 256


typedef KGenericFactory<KDataSources> KDataSourcesFactory;
K_EXPORT_COMPONENT_FACTORY(datasources, KDataSourcesFactory("kcm_datasources"))


KDataSources::KDataSources(QWidget *parent, const QStringList &args) : KCModule(KDataSourcesFactory::componentData(), parent, args)
{
	// Variables...
	QStringList header;

	// About...
	KAboutData *about = new KAboutData("kcm_datasources", 0, ki18n("Data Source Configuration Module"), KDE_VERSION_STRING, KLocalizedString(), KAboutData::License_GPL, ki18n("Copyright 2007 Ian Ventura-Whiting"));
	about->addAuthor(ki18n("Ian Ventura-Whiting"), KLocalizedString(), "fizz@titania.co.uk");
	setAboutData(about);

	// Quick Help...
	setQuickHelp( i18n("<h1>Data Source Configuration</h1>Help here"));

	// Main Data Sources Layout...
	QGridLayout *mainLayout = new QGridLayout(this);
	mainLayout->setMargin(0);
	mainLayout->setSpacing(KDialog::spacingHint());

	// Data Source List...
	dataSourceList = new QTreeWidget(this);
	mainLayout->addWidget(dataSourceList, 0, 0, 5, 1, 0);
	dataSourceList->setColumnCount(3);
	header.clear();
	header << i18n("Name");
	header << i18n("Description");
	header << i18n("Type");
	header << i18n("Driver");
	dataSourceList->setHeaderLabels(header);
	dataSourceList->setRootIsDecorated(false);
	dataSourceList->setItemsExpandable(false);
	dataSourceList->setSelectionBehavior(QAbstractItemView::SelectRows);
	dataSourceList->setSelectionMode(QAbstractItemView::SingleSelection);

	// Add Button...
	addDSNButton = new KPushButton(this);
	mainLayout->addWidget(addDSNButton, 0, 1, 0);
	addDSNButton->setText(i18n("Add"));
	addDSNButton->setIcon(KIcon("list-add"));

	// Modify Button...
	modifyDSNButton = new KPushButton(this);
	mainLayout->addWidget(modifyDSNButton, 1, 1, 0);
	modifyDSNButton->setText(i18n("Modify"));
	modifyDSNButton->setEnabled(FALSE);
	modifyDSNButton->setIcon(KIcon("document-properties"));

	// Remove Button...
	removeDSNButton = new KPushButton(this);
	mainLayout->addWidget(removeDSNButton, 2, 1, 0);
	removeDSNButton->setText(i18n("Remove"));
	removeDSNButton->setEnabled(FALSE);
	removeDSNButton->setIcon(KIcon("edit-delete"));

	// Test Button...
	testDSNButton = new KPushButton(this);
	mainLayout->addWidget(testDSNButton, 3, 1, 0);
	testDSNButton->setText(i18n("Test"));
	testDSNButton->setEnabled(false);
	testDSNButton->setIcon(KIcon("system-run"));

	// Button Spacer...
	QSpacerItem *mainButtonSpacer = new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding);
	mainLayout->addItem(mainButtonSpacer, 4, 1);

	// Load
	load();

	// Signals and Slots
	connect(addDSNButton, SIGNAL(clicked(bool)), this, SLOT(addDSN()));
	connect(removeDSNButton, SIGNAL(clicked(bool)), this, SLOT(removeDSN(void)));
	connect(dataSourceList, SIGNAL(itemSelectionChanged()), this, SLOT(dsnSelectionChanged()));
}


KDataSources::~KDataSources()
{
}


// Load list of DSN
void KDataSources::load()
{
	// Clear the List...
	dataSourceList->clear();

	// Update the list...
	updateDSNList(ODBC_USER_DSN);			// User DSN
	updateDSNList(ODBC_SYSTEM_DSN);			// System DSN
	updateFileDSNList();					// File DSN
}


// DSN Selection Changed
void KDataSources::dsnSelectionChanged()
{
	// Variables....
	bool itemsSelected;
	bool isRoot;

	// Are any DSN selected?
	if (dataSourceList->selectedItems().count() > 0)
		itemsSelected = TRUE;
	else
		itemsSelected = FALSE;

	// Is the user root?
	if (getuid() == 0)
		isRoot = TRUE;
	else
		isRoot = FALSE;

	// Modify Button...
	if (itemsSelected == FALSE)
		modifyDSNButton->setEnabled(FALSE);
	else if (isRoot == TRUE)
	{
		modifyDSNButton->setEnabled(TRUE);
		modifyDSNButton->setText(i18n("Modify"));
	}
	else if (dataSourceList->currentItem()->text(2) == i18n("User"))
	{
		modifyDSNButton->setEnabled(TRUE);
		modifyDSNButton->setText(i18n("Modify"));
	}
	else
	{
		modifyDSNButton->setEnabled(TRUE);
		modifyDSNButton->setText(i18n("View"));
	}

	// Delete Button...
	if (itemsSelected == FALSE)
		removeDSNButton->setEnabled(FALSE);
	else if (isRoot == TRUE)
		removeDSNButton->setEnabled(TRUE);
	else if (dataSourceList->currentItem()->text(2) == i18n("User"))
		removeDSNButton->setEnabled(TRUE);
	else
		removeDSNButton->setEnabled(FALSE);

	// Test Button...
	if (itemsSelected == FALSE)
		testDSNButton->setEnabled(FALSE);
	else
		testDSNButton->setEnabled(TRUE);
}


// Add DSN
void KDataSources::addDSN()
{
	DSNDialog *addDataSource = new DSNDialog(this);
	addDataSource->setup();
	if (addDataSource->exec() == QDialog::Accepted)
		load();
}


// Delete DSN
void KDataSources::removeDSN()
{
	// Variables...
	QString tempString;

	// Set SQL Config Mode...
	if (dataSourceList->currentItem()->text(2) == i18n("User"))
		SQLSetConfigMode(ODBC_USER_DSN);
	else if (dataSourceList->currentItem()->text(2) == i18n("System"))
		SQLSetConfigMode(ODBC_SYSTEM_DSN);

	// Build Dialog Question...
	tempString = QString(i18n("Are you sure you want to delete the data source?")).arg(dataSourceList->currentItem()->text(0));

	// Ask confirmation...
	if (KMessageBox::questionYesNo(this, tempString, i18n("Remove Data Source")) == KMessageBox::Yes)
	{
		// Delete data Source...
		if (SQLRemoveDSNFromIni(dataSourceList->currentItem()->text(0).toLatin1()) == FALSE)
		{
			KMessageBox::error(this, i18n("Could not remove the data source."), i18n("Remove Data Source"), KMessageBox::PlainCaption);
		}
		else
		{
			load();
		}
	}

	// Reset SQL Config Mode...
	SQLSetConfigMode(ODBC_BOTH_DSN);
}


// Update User and System DSNs
void KDataSources::updateDSNList(int dsnType)
{
	// Variables...
	char odbcResults[ODBC_BUFFER_SIZE + 1];
	char odbcDescResults[ODBC_BUFFER_SIZE + 1];
	QString odbcName;
	QTreeWidgetItem *currentItem = 0;
	int odbcResultsPos = 0;

	// Populate the list...
	SQLSetConfigMode(dsnType);
	if (SQLGetPrivateProfileString(0, 0, 0, odbcResults, ODBC_BUFFER_SIZE, 0) > 0)
	{
		odbcName = QString(odbcResults);
		while (!odbcName.isEmpty())
		{
			// Create item...
			currentItem = new QTreeWidgetItem(dataSourceList);

			// Set text...
			currentItem->setText(0, odbcName.toLatin1());
			SQLGetPrivateProfileString(odbcName.toLatin1(), "Description", "", odbcDescResults, ODBC_BUFFER_SIZE, 0);
			currentItem->setText(1, QString(odbcDescResults));
			if (dsnType == ODBC_USER_DSN)
				currentItem->setText(2, i18n("User"));
			else
				currentItem->setText(2, i18n("System"));
			SQLGetPrivateProfileString(odbcName.toLatin1(), "Driver", "", odbcDescResults, ODBC_BUFFER_SIZE, 0);
			currentItem->setText(3, QString(odbcDescResults));

			// Get the next driver name...
			odbcResultsPos = odbcResultsPos + odbcName.length() + 1;
			if (odbcResultsPos < ODBC_BUFFER_SIZE)
				odbcName = QString(odbcResults + odbcResultsPos);
			else
				odbcName = "";
		}
	}
	SQLSetConfigMode(ODBC_BOTH_DSN);
}


// Update the File DSNs...
void KDataSources::updateFileDSNList()
{
	// Declare Variables...
	QTreeWidgetItem *currentItem = 0;
	QFileInfoList directoryFileList;
	QDir directory;
	QStringList fileDSNFilter;
	char odbcResults[ODBC_BUFFER_SIZE + 1];
	char fileDSNPath[ODBC_FILEDSN_SIZE];
	char fileDSNName[ODBC_FILEDSN_SIZE];

	// Set File DSN dir...
	if (QFile::exists("/etc/unixODBC/ODBCDataSources/"))
		sprintf(fileDSNName, "/etc/unixODBC/ODBCDataSources/");
	else
		sprintf(fileDSNName, "/");
	SQLGetPrivateProfileString("ODBC", "FileDSNPath", fileDSNName, fileDSNPath, ODBC_FILEDSN_SIZE, "odbcinst.ini");

	// Get file list...
	directory.setFilter(QDir::Files | QDir::Hidden);
	fileDSNFilter << "*.dsn";
	directory.setNameFilters(fileDSNFilter);
	directory.cd(fileDSNPath);
	directoryFileList = directory.entryInfoList();

	// Process file list...
	for (int fileLoop = 0; fileLoop < directoryFileList.size(); fileLoop++)
	{
		QFileInfo fileDSN = directoryFileList.at(fileLoop);
		currentItem = new QTreeWidgetItem(dataSourceList);
		currentItem->setText(0, fileDSN.baseName());
		if (SQLReadFileDSN(fileDSN.fileName().toLatin1(), "ODBC", "Description", odbcResults, ODBC_BUFFER_SIZE, NULL) == 0 || strlen(odbcResults) == 0)
			currentItem->setText(1, "");
		else
			currentItem->setText(1, QString(odbcResults));
		currentItem->setText(2, i18n("File"));
		if (SQLReadFileDSN(fileDSN.fileName().toLatin1(), "ODBC", "Driver", odbcResults, ODBC_BUFFER_SIZE, NULL) == 0 || strlen(odbcResults) == 0)
			currentItem->setText(3, "");
		else
			currentItem->setText(3, QString(odbcResults));
	}
}


