/*  This file is part of the KDE project
    Copyright (C) 2006 Kevin Ottens <ervin@kde.org>
    Copyright (C) 2009 Dario Freddi <drf54321@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.

*/

#ifndef SOLIDBACKENDCHOOSER_H
#define SOLIDBACKENDCHOOSER_H

#include <QWidget>
#include <QString>

#include <KService>

class SolidBackendChooserPrivate;

/**
 *  @short %SOLID backend chooser widget
 *
 *  Lets a user select a backend for Solid
 *
 *  This widget is able to display available backends for a specified
 *  Solid backend type. It lets the user define the order in which
 *  the backends will be preferred.
 *
 *  @since 4.3
 *
 *  @author Kevin Ottens <ervin@kde.org> and Dario Freddi <drf54321@gmail.com>
 */
class SolidBackendChooser : public QWidget
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(SolidBackendChooser)
    Q_DISABLE_COPY(SolidBackendChooser)
public:
    /**
     *  Constructs a new SolidBackendChooser.
     *
     *  \param backendType the backend type we want to display
     *  \param parent the widget parent
     */
    explicit SolidBackendChooser(const QString &backendType = QString(), QWidget *parent = 0);
    /**
     *  Destructor.
     *
     *  Deletes all internal objects.
     */
    ~SolidBackendChooser();

    /**
     *  Saves the current selection
     */
    void save();
    /**
     *  Restores the defaults in the widget
     */
    void defaults();

    /**
     *  Sets the backend type the widget will let the user select
     *  the backend for. After issuing this call, the widget will
     *  be automatically reloaded.
     *
     *  \param type the new backend type
     */
    void setBackendType(const QString &type);
    /**
     *  Returns the current backend type used by the widget
     */
    QString backendType() const;

    /**
     * Provides access to the currently selected backend KService
     */
    KService::Ptr selectedBackend();

Q_SIGNALS:
    /**
     *  Emitted when the selection in the widget gets changed.
     *
     *  \param differsFromDefault if \c false, the widget is in its default state.
     *                            Otherwise, the selection has been changed by the user.
     */
    void currentSelectionChanged(bool wasOriginallySelected);
    /**
     *  Emitted when the current backend type changes
     *
     *  \param type the new backend type
     *
     *  \see setBackendType
     */
    void backendTypeChanged(const QString &type);

private:
    SolidBackendChooserPrivate *d_ptr;

    Q_PRIVATE_SLOT(d_func(), void slotSelectionChanged())
    Q_PRIVATE_SLOT(d_func(), void slotUpClicked())
    Q_PRIVATE_SLOT(d_func(), void slotDownClicked())
};

#endif
