/*
 * KDE Composite Manager, 
 * Copyright (C) 2003-2005 Fredrik Höglund <fredrik@kde.org>
 *
 * Based on xcompmgr,
 * Copyright (C) 2003 Keith Packard <keithp@keithp.com>
 *
 * Internal structure and naming style inspired by KWin,
 * Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 * Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Fredrik Höglund not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Fredrik Höglund makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * FREDRIK HÖGLUND DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL FREDRIK HÖGLUND BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef CLIENT_H
#define CLIENT_H

#include <QRect>
#include <netwm_def.h>
#include "workspace.h"

enum ShapeType { WindowOnly, WindowAndBorder, WindowAndShadow, Shadow };

class Client
{
	public:
		
		// Creates a client object for a window.
		Client( Window id, const XWindowAttributes & );

		 // Destroys the client object.
		~Client();

		// Returns the window ID for the client.
		Window winId() const { return mId; }

		// Returns the ID of the embedded window, if the client is a WM frame.
		// If not, the returned ID is the same as the window ID.
		Window clientId() const { return mClientId; }

		 // Returns a handle to the damage object for the client.
		Damage damageHandle() const { return mDamage; }

		// Returns a handle to the Xrender picture for the window pixmap.
		// The picture handle will be created if it doesn't exist.
		Picture picture() const;

		// Returns a handle to the Xrender picture for the shadow.
		// The shadow picture will be created if it doesn't exist.
		Picture shadow() const;

		// Derefs the window pixmap and frees the Xrender picture,
		// forcing them to be recreated in the next paint cycle.
		// (or when picture() is called)
		void renewWindowPixmap();

		// Returns true if the window pixmap has an alpha channel,
		// and false otherwise
		bool hasAlphaChannel() const { return mAlphaChannel; }

		// Returns true if the client is opaque (i.e. it doesn't have an
		// alpha channel, and its opacity is set to 1.0).
		bool isOpaque() const;

		// Returns true if the client is override redirect, and false otherwise.
		bool isOverrideRedirect() const { return mOverrideRedirect; }

		// Returns true if the client is input-only, and false otherwise.
		bool isInputOnly() const { return mInputOnly; }

		// Returns true if the client is a transient window.
		bool isTransient() const { return ( mTransient != None ); }

		// Sets the window for which this client is transient.
		void setTransient( Window w ) { mTransient = w; }

		// Creates a region object for the specified type.
		XserverRegion createRegion( ShapeType type ) const;

		// Returns a cached shape region object matching the window shape
		// (As in XShape) precisely. The region is relative to the screen.
		XserverRegion shape() const;

		// Updates the region object holding the region where the client was
		// last rendered. Called by Workspace::repaint().
		void updateOnScreenRegion();

		// Creates a copy of the specified region, and saves it in the client
		// object. Called by Workspace::repaint().
		void saveClipRegion( const XserverRegion );

		// Returns the region saved by saveClipRegion().
		XserverRegion savedClipRegion() const { return mSavedClipRegion; }

		// Destroys the region saved by saveClipRegion().
		void destroySavedClipRegion();

		void reconfigure();

		int x() const { return mGeometry.x(); }
		int y() const { return mGeometry.y(); }
		int width() const { return mGeometry.width(); }
		int height() const { return mGeometry.height(); }
		int borderWidth() const { return mBorderWidth; }
		QRect geometry( ShapeType ) const;
		Picture alphaMask() const;
		void setOpacityHintValid( bool v ) { mOpacityHintValid = mAlphaMaskValid = v; }
		double opacity() const;
		double shadowRadius() const { return mShadowRadius; }

		// Used by animations to set the opacity relative to the opacity hint.
		void setAnimationOpacity( double opacity );

		// Returns the animation opacity.
		double animationOpacity() const { return mOpacity; }

		// Returns the value of EWMH window type hint.
		// The value will be read from embedded window if necessary.
		NET::WindowType windowType() const;

		void setWindowTypeValid( bool v ) { mWindowTypeValid = v; }

		// Returns the number of the virtual desktop this client is on.
		// If the window is on all desktops, NET::OnAllDesktops is returned.
		int desktop() const { return mDesktop; }

		// Sets the number of the virtual desktop this client is on.
		// Note: this doesn't move the client to specified desktop.
		void setDesktop( int desktop ) { mDesktop = desktop; }

		void setPainted( bool painted ) { mPainted = painted; }

		// These functions are called by Workspace to tell the client something
		// has changed, and it should take action.
		void geometryChanged( const QRect &newGeometry, int newBorderWidth );
		void shapeChanged();
		void opacityChanged();
		void transientForChanged();
		void focusIn();
		void focusOut();

		void show();
		void hide();

		// Returns true if the client is visible (i.e. mapped) or false otherwise
		bool isVisible() const { return mVisible; }

		// Returns true if the window has been painted since it was last shown (mapped).
		bool isPainted() const { return mPainted; }

		// Returns true if this client is the active window
		bool isActive() const { return mActive; }


	private:
		Picture createWindowPicture() const;
		Picture createShadowPicture() const;
		void updateAlphaMask() const;

		void setHasAlphaChannel( bool value ) { mAlphaChannel = value; }
		void setShadowRadius( double radius );

		double getOpacityHint() const;
		NET::WindowType getWindowType() const;

		Window                  mId;                   // Window ID
		Window                  mClientId;             // For WM frames, the ID of the embedded window
		Window                  mTransient;            // The window for which mClientId is transient
		XRenderPictFormat      *mFormat;               // The Render format of the window pixmap
		mutable Pixmap          mPixmap;               // The window pixmap
		mutable Picture         mPicture;              // The Window picture
		mutable Picture         mShadowPict;           // Cached shadow picture
		mutable Picture         mAlphaMask;            // 1x1 repeating alpha mask picture
		mutable double          mOpacity;              // Opacity relative to mOpacityHint
		mutable double          mOpacityHint;          // Value of the opacity hint
		Damage                  mDamage;               // Damage handle
		QRect                   mGeometry;             // Window geometry
		int                     mBorderWidth;          // X border size
		XserverRegion           mVisibleRegion;        // Cached WindowAndShadow region
		mutable XserverRegion   mShapeRegion;          // Cached Window region
		XserverRegion           mSavedClipRegion;      // Clip region saved by Workspace
		mutable NET::WindowType mWindowType;           // EWMH window type
		int                     mDesktop;              // Virtual desktop number
		double                  mShadowRadius;         // Window drop shadow radius
		bool                    mVisible:1;            // Whether the window is mapped
		bool                    mPainted:1;            // Whether the window has been painted
		bool                    mActive:1;             // Whether the window is the active window
		mutable bool            mShapeClipValid:1;     // Whether the mPicture clip region is valid
		bool                    mAlphaChannel:1;       // Whether mPicture has an alpha channel
		bool                    mOverrideRedirect:1;   // Whether the window is override redirect
		bool                    mInputOnly:1;          // Whether the window is input only
		mutable bool            mWindowTypeValid:1;    // Whether mWindowType is valid
		mutable bool            mAlphaMaskValid:1;     // Whether mAlphaMask is valid
		mutable bool            mOpacityHintValid:1;   // Whether mOpacityHint is valid
};


inline void Client::setAnimationOpacity( double opacity )
{
	if ( int(opacity * 0xff) != int(mOpacity * 0xff) )
		mAlphaMaskValid = false;

	mOpacity = opacity;
}


inline bool Client::isOpaque() const
{
	if ( hasAlphaChannel() || opacity() < 1.0 )
		return false;

	return true;
}


inline double Client::opacity() const
{
	if ( !mOpacityHintValid ) {
		mOpacityHint = getOpacityHint();
		mOpacityHintValid = true;
	}

	return mOpacity * mOpacityHint;
}


inline Picture Client::picture() const
{
	// The render format can be null for GL and/or Xv visuals
	if ( !mFormat )
		return None;

	if ( !mPicture )
		mPicture = createWindowPicture();

	// ### There's room for optimization here. The window picture only needs a clip region when it's
	// translucent. For opaque windows it doesn't matter since we're drawing them top -> bottom.
	if ( !mShapeClipValid /*&& !isOpaque()*/ ) {
		XserverRegion clip = XFixesCreateRegionFromWindow( dpy, winId(), WindowRegionBounding );
		XFixesSetPictureClipRegion( dpy, mPicture, 0, 0, clip );
		XFixesDestroyRegion( dpy, clip );
		mShapeClipValid = true;
	}

	return mPicture;
}


inline Picture Client::shadow() const
{
	if ( !mShadowPict )
		mShadowPict = createShadowPicture();

	return mShadowPict;
}

inline NET::WindowType Client::windowType() const
{
	if ( !mWindowTypeValid ) {
		mWindowType = getWindowType();
		mWindowTypeValid = true;	
	}

	return mWindowType;
}


inline Picture Client::alphaMask() const
{
	if ( !mAlphaMaskValid )
		updateAlphaMask();

	return mAlphaMask;
}


inline void Client::destroySavedClipRegion()
{
	if ( mSavedClipRegion ) {
		XFixesDestroyRegion( dpy, mSavedClipRegion );
		mSavedClipRegion = None;
	}
}


inline void Client::saveClipRegion( const XserverRegion r )
{
	if ( !mSavedClipRegion )
		mSavedClipRegion = XFixesCreateRegion( dpy, 0, 0 );
	XFixesCopyRegion( dpy, mSavedClipRegion, r );
}

#endif

