/*
    This file is part of KDE.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "appmanager.h"

#include "entry.h"
#include "engine.h"
#include "installer.h"

#include <kmessagebox.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <ksimpleconfig.h>
#include <kdesktopfile.h>

AppManager::AppManager( Engine *engine, Installer *installer )
  : DCOPObject( "AppManager" ), mEngine( engine ), mInstaller( installer )
{
}

QString AppManager::installedVersion( const QString &appname )
{
  kdDebug() << "APPNAME: " << appname << endl;

  KDesktopFile d( appname );
  kdDebug() << "DESKTOP: " << d.readName() << endl;

  QString installed = locate( "appdata", "installed/" + appname +
    ".kinstaller" );
  kdDebug() << "INSTALLED: " << installed << endl;

  if ( installed.isEmpty() ) return QString::null;

  KSimpleConfig cfg( installed );
  cfg.setGroup( "Application" );
  QString version = cfg.readEntry( "Version" );
  return version;
}

QString AppManager::newestVersion( const QString &appname )
{
  QString newest;

  KNS::Entry::List entries = mEngine->entriesForName( appname );
  KNS::Entry::List::ConstIterator it;
  KNS::Entry::List::ConstIterator end( entries.end() );
  for( it = entries.begin(); it != end; ++it ) {
    if ( compareVersion( (*it)->version(), newest ) == 1 ) {
      newest = (*it)->version();
    }
  }
  return newest;
}

bool AppManager::hasNewerVersion( const QString &appname,
  const QString &version )
{
  KNS::Entry::List entries = mEngine->entriesForName( appname );
  KNS::Entry::List::ConstIterator it;
  KNS::Entry::List::ConstIterator end( entries.end() );
  for( it = entries.begin(); it != end; ++it ) {
    if ( compareVersion( (*it)->version(), version ) == 1 ) return true;
  }
  return false;
}

int AppManager::compareVersion( const QString &v1, const QString &v2 )
{
  QStringList elements1 = QStringList::split( ".", v1 );
  QStringList elements2 = QStringList::split( ".", v2 );
  uint max = QMAX( elements1.size(), elements2.size() );
  uint i;
  for( i = 0; i < max; ++i ) {
    QString e1;
    if ( elements1.size() > i ) e1 = elements1[ i ];
    else e1 = "0";
    int n1 = e1.toInt();

    QString e2;
    if ( elements2.size() > i ) e2 = elements2[ i ];
    else e2 = "0";
    int n2 = e2.toInt();

    if ( n1 > n2 ) return 1;
    if ( n1 < n2 ) return -1;
  }
  return 0;
}

void AppManager::install( const QString &appname, const QString &version )
{
  KNS::Entry::List entries = mEngine->entriesForName( appname );
  KNS::Entry::List::ConstIterator it;
  KNS::Entry::List::ConstIterator end(entries.end() );

  for( it = entries.begin(); it != end; ++it ) {
    if ( (*it)->version() == version ) {
      mInstaller->install( *it );
      break;
    }
  }
  if ( it == entries.end() ) {
    KMessageBox::sorry( 0, i18n("'%1' Version '%2' not found.").arg( appname )
      .arg( version ) );
  }
}

void AppManager::update( const QString &appname )
{
  QString newest = newestVersion( appname );

  if ( compareVersion( newest, installedVersion( appname ) ) == 1 ) {
    install( appname, newestVersion( appname ) );
  }
}
