/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "klipboard.h"

#include <KApplication>
#include <KDebug>

#include <QClipboard>
#include <QMimeData>
#include <QUuid>

#include "clipboardpoll.h"
#include "historyfile.h"

class KlipboardSingletonPrivate
{
public:
    Klipboard instance;
};
K_GLOBAL_STATIC(KlipboardSingletonPrivate, klipboardSingleton)

class KlipboardPrivate
{
public:
    KlipboardPrivate(Klipboard* klipboard)
        : clipPoll(0), historySize(0), m_klipboard(klipboard)
    {}

    ClipboardPoll* clipPoll;
    ClipboardEntryList history;
    int historySize;

    HistoryFile historyFile;

    /**
     * Adds a new clipboard entry to history while respecting history size.
     * i.e. it can pop some items at bottom
     */
    void addEntry(const ClipboardEntry& entry, bool saveToHistoryFile);

    /**
     * Removes a specified entry from history
     */
    void removeEntry(const ClipboardEntry& entry);

    /**
     * Removes all occurances of entry with the specified uuid
     */
    void removeEntry(const QString& uuid);

    void populateHistory(const ClipboardEntryList& entryList, bool saveToHistoryFile);

private:
    Klipboard* m_klipboard;
};

Klipboard* Klipboard::self()
{
    return &klipboardSingleton->instance;
}

Klipboard::Klipboard()
    : d(new KlipboardPrivate(this))
{
    d->clipPoll = new ClipboardPoll;
    connect(d->clipPoll, SIGNAL(clipboardChanged(bool)), SLOT(onClipboardChanged(bool)));
}

Klipboard::~Klipboard()
{
    delete d->clipPoll;
    delete d;
}

void Klipboard::setHistorySize(int size)
{
    d->historySize = size;
}

int Klipboard::historySize() const
{
    return d->historySize;
}

void Klipboard::setHistoryFile(const QString& filename)
{
    if (d->historyFile.isWritable())
        d->historyFile.close();

    clearHistory();
    bool res = d->historyFile.setFile(filename, HistoryFile::CreateIfAbsentMode);
    if (!res)
    {
        kDebug() << "failed to set history file";
        return;
    }

    d->populateHistory(d->historyFile.entries(), false);
}

int Klipboard::entryCount() const
{
    return d->history.count();
}

bool Klipboard::isHistoryEmpty() const
{
    return d->history.isEmpty();
}

void Klipboard::setCurrentEntry(const QString& uuid)
{
    ClipboardEntry e = entry(uuid);
    d->removeEntry(uuid);

    QClipboard::Mode mode = e.isSelection ? QClipboard::Selection : QClipboard::Clipboard;

    if (e.isText)
        kapp->clipboard()->setText(e.data.toString(), mode);
    else
        kapp->clipboard()->setImage(e.data.value<QImage>(), mode);

    //addEntry(e);
}

ClipboardEntry Klipboard::currentEntry() const
{
    return d->history.last();
}


void Klipboard::clearHistory()
{
    foreach (const ClipboardEntry& entry, d->history)
        emit entryRemoved(entry.uuid);

    d->history.clear();
}

void Klipboard::populateHistory(const ClipboardEntryList& entryList)
{
    d->populateHistory(entryList, true);
}

ClipboardEntry Klipboard::entry(const QString& uuid) const
{
    foreach (const ClipboardEntry& entry, d->history)
    {
        if (entry.uuid == uuid)
            return entry;
    }

    return ClipboardEntry();
}

ClipboardEntry Klipboard::entry(int index) const
{
    return d->history.at(index);
}

void Klipboard::onClipboardChanged(bool selectionMode)
{
    kDebug() << "clilpboard changed";
    const QMimeData* mime = kapp->clipboard()->mimeData( selectionMode ? QClipboard::Selection : QClipboard::Clipboard);
    if (!mime) {
        kDebug() << "error: mime data object is null";
        return;
    }

    ClipboardEntry entry;
    if (mime->hasText())
    {
        entry.data = mime->text();
        entry.uuid = QUuid::createUuid();
        entry.isSelection = selectionMode;
        entry.isText = true;
    }
    else if (mime->hasImage())
    {
        entry.data = mime->imageData();
        entry.uuid = QUuid::createUuid();
        entry.isSelection = selectionMode;
        entry.isText = false;
    }

    if (entry.isValid())
        d->addEntry(entry, true);
}

void KlipboardPrivate::addEntry(const ClipboardEntry& entry, bool saveToHistoryFile)
{
    // do not allow empty entries
    if (entry.isText && entry.data.toString().isEmpty())
    {
        kDebug() << "rejecting addition of empty clipboard content";
        return;
    }

    // do not allow duplicates
    // TODO check that this works ok for large texts/images.
    // perhaps it needs an optimization of comparison operation (somehow)
    if ( !history.isEmpty() &&
         entry.data == history.last().data &&
         entry.isSelection == history.last().isSelection )
    {
        kDebug() << "rejecting addition of duplicated clipboard content";
        return;
    }

    ClipboardEntry newEntry(entry);
    if (entry.uuid.isEmpty())
        newEntry.uuid = QUuid::createUuid();

    history.append(newEntry);
    if (saveToHistoryFile)
        historyFile.addEntry(entry);

    emit m_klipboard->entryAdded(newEntry.uuid);
    emit m_klipboard->currentEntryChanged(newEntry.uuid);

    // do not allow to overfill
    if (history.count() > historySize)
        removeEntry(history.first());
}

void KlipboardPrivate::removeEntry(const ClipboardEntry& entry)
{
    QString uuid = entry.uuid;
    int numRemoved = history.removeAll(entry);
    bool res = historyFile.removeEntry(entry.uuid);
    if (res == false)
        kDebug() << "failed to remove entry" << entry.data.toString() << "from history file";

    if (numRemoved != 0)
        emit m_klipboard->entryRemoved(uuid);
}

void KlipboardPrivate::removeEntry(const QString& uuid)
{
    QList<int> idxToRemove;
    int count = history.count();
    for (int i=0; i<count; ++i)
    {
        if (history.at(i).uuid == uuid)
            idxToRemove.append(i);
    }

    foreach (int idx, idxToRemove)
    {
        QString uuid = history[idx].uuid;
        history.removeAt(idx);
        emit m_klipboard->entryRemoved(uuid);
    }
}

void KlipboardPrivate::populateHistory(const ClipboardEntryList& entryList, bool saveToHistoryFile)
{
    foreach (const ClipboardEntry& entry, entryList)
        addEntry(entry, saveToHistoryFile);
}
