/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "klipperwidget.h"

#include <QListView>
#include <QVBoxLayout>
#include <QFocusEvent>

#include <Plasma/Service>
#include <Plasma/Delegate>

#include "commondefs.h"

#include "klipboardmodel.h"
#include "klipboarddelegate.h"

KlipperWidget::KlipperWidget(QWidget* parent)
    : QWidget(parent), m_engine(0), m_model(0)
{
    m_listView = new QListView(this);
// XXX: temporary for debugging focus problems
    m_listView->installEventFilter(this);

    // needed to correctly pass focus to list view
    setFocusPolicy(Qt::WheelFocus);
    setFocusProxy(m_listView);

    QPalette p(palette());
    p.setColor(QPalette::Base, Qt::transparent);
    m_listView->setPalette(p);
    setAttribute(Qt::WA_NoSystemBackground, true);
    setAutoFillBackground(true);

    m_listView->viewport()->setAutoFillBackground(true);
    m_listView->viewport()->setAttribute(Qt::WA_NoSystemBackground, true);
    m_listView->setFrameShape(QFrame::NoFrame);

    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->addWidget(m_listView);

    connect(m_listView, SIGNAL(activated(const QModelIndex&)),
            SLOT(onActivated(const QModelIndex&)));
}

void KlipperWidget::setClipboardEngine(Plasma::DataEngine* engine)
{
    if (!engine || !engine->isValid())
    {
        kDebug() << "Null or not valid data engine passed";
        return;
    }

    m_engine = engine;

    // we want to get updates on current history entry changes from dataengine
    // if source isn't available yet, watch for its addition and then connect
    if (m_engine->sources().contains(CURRENT_ENTRY_SOURCE))
    {
        kDebug() << "connecting to" << CURRENT_ENTRY_SOURCE << "source";
        m_engine->connectSource(CURRENT_ENTRY_SOURCE, this);
    }
    else
    {
        connect(m_engine, SIGNAL(sourceAdded(const QString&)),
                SLOT(onSourceAdded(const QString&)));
    }

    m_model = new KlipboardModel(m_engine, this);
    KlipboardDelegate* delegate = new KlipboardDelegate(this);
    delegate->setRoleMapping( Plasma::Delegate::SubTitleRole, KlipboardModel::ClipboardTypeRole );
    delegate->setRoleMapping( Plasma::Delegate::SubTitleMandatoryRole, KlipboardModel::AlwaysShowClipboardTypeRole );
    m_listView->setModel(m_model);
    m_listView->setItemDelegate(delegate);
}

void KlipperWidget::onActivated(const QModelIndex& index)
{
    if (!m_engine)
    {
        kDebug() << "Null or not valid data engine";
        return;
    }

    if (!m_model)
    {
        kDebug() << "Error: model object is null";
        return;
    }

    Plasma::Service* service = m_engine->serviceForSource("klipper");
    KConfigGroup operation = service->operationDescription("setCurrentEntry");
    operation.writeEntry("uuid", m_model->data(index, KlipboardModel::UuidRole ));
    service->startOperationCall(operation);

    emit currentEntryChanged();
}

void KlipperWidget::onSourceAdded(const QString& source)
{
    if (source == CURRENT_ENTRY_SOURCE)
    {
        kDebug() << "connecting to" << CURRENT_ENTRY_SOURCE << "source";
        m_engine->connectSource(CURRENT_ENTRY_SOURCE, this);

        // now that we connected, we aren't interested in notifications on new sources
        disconnect(m_engine, SIGNAL(sourceAdded(const QString&)),
                   this, SLOT(onSourceAdded(const QString&)));
    }
}

void KlipperWidget::dataUpdated(const QString& sourceName, const Plasma::DataEngine::Data& data )
{
    if (!m_model)
    {
        kDebug() << "Error: model object is null";
        return;
    }

    // sourceName will be CURRENT_ENTRY_SOURCE, value will be the uuid of selected entry

    kDebug() << "dataUpdated" << data.value(sourceName).toString();
    QModelIndex index = m_model->indexForUuid(data.value(sourceName).toString());
    if (index.isValid())
        m_listView->setCurrentIndex(index);
    else
        kDebug() << "Warning: invalid index occured when trying to set current entry";
}

// XXX: temporary for debugging focus problems
bool KlipperWidget::eventFilter(QObject* obj, QEvent* ev)
{
    if (obj == m_listView)
    {
        if (ev->type() == QEvent::FocusIn)
        {
            kDebug() << "Focus In," << "reason:" << static_cast<QFocusEvent*>(ev)->reason();
        }
        else if (ev->type() == QEvent::FocusOut)
        {
            kDebug() << "Focus Out," << "reason:" << static_cast<QFocusEvent*>(ev)->reason();
        }
    }

    return false;
}

#include "klipperwidget.moc"
