/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "syncrequest.h"
#include "syncrequest_p.h"
#include "nie.h"
#include "nfo.h"
#include "backupsync.h"
#include "logstorage.h"

#include "changelog.h"
#include "changelogrecord.h"
#include "graphwrapper_p.h"

#include <algorithm>

#include <QtCore/QMultiHash>
#include <QtCore/QHashIterator>
#include <QtCore/QThread>

#include <Soprano/Node>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/NRL>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Statement>
#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>
#include <Soprano/NodeIterator>

#include <Nepomuk/Resource>
#include <Nepomuk/Variant>
#include <Nepomuk/Types/Property>
#include <Nepomuk/ResourceManager>

#include <KDebug>

int Nepomuk::SyncRequest::Private::NextId = 0;

Nepomuk::SyncRequest::SyncRequest(const Nepomuk::Sync::ChangeLog& log )
    : d( new Nepomuk::SyncRequest::Private )
{
    d->m_id = Private::NextId++;
    d->m_logFile = log;
    d->m_model = ResourceManager::instance()->mainModel();

    ResourceManager * rm = ResourceManager::instance();
    //if( model != ResourceManager::instance()->mainModel() )
    //    rm = ResourceManager::createManagerForModel( model );

    d->m_graphWrapper.setManager( rm );
}


Nepomuk::SyncRequest::SyncRequest()
    : d( new Nepomuk::SyncRequest::Private )
{
    d->m_id = -1;
    d->m_model = ResourceManager::instance()->mainModel();

    ResourceManager * rm = ResourceManager::instance();
    //if( model != ResourceManager::instance()->mainModel() )
    //    rm = ResourceManager::createManagerForModel( model );

    d->m_graphWrapper.setManager( rm );
}


Nepomuk::SyncRequest::~SyncRequest()
{
}


void Nepomuk::SyncRequest::load()
{
    if( d->m_logFile.empty() )
        return;

    if( !d->m_hash.isEmpty() )
        return;

    d->m_hash = Sync::createResourceLogMap( d->m_logFile );

    // The records are stored according to dateTime
    d->m_minDateTime = d->m_logFile.toList().first().dateTime();
}



int Nepomuk::SyncRequest::id() const
{
    return d->m_id;
}


bool Nepomuk::SyncRequest::done() const
{
    return d->m_jobs.empty();
}


namespace {

    //
    // Cache the results. This could have very bad consequences if someone updates the ontology
    // when the service is running
    //
    static QSet<QUrl> nonMergeable;
    bool isMergeable( const QUrl & prop, Soprano::Model * model ) {

        if( nonMergeable.contains( prop ) )
            return false;

        QString query = QString::fromLatin1( "ask { %1 %2 false . }" )
        .arg( Soprano::Node::resourceToN3( prop ) )
        .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::backupsync::mergeable() ) );

        bool isMergeable = !model->executeQuery( query, Soprano::Query::QueryLanguageSparql ).boolValue();

        if( !isMergeable ) {
            nonMergeable.insert( prop );
            return false;
        }
        return true;
    }

    QList<Nepomuk::Sync::ChangeLogRecord> getRecords( const Nepomuk::Sync::ResourceLogMap & hash, const QUrl resUri, const QUrl & propUri ) {

        Nepomuk::Sync::ResourceLogMap::const_iterator it = hash.constFind( resUri );
        if( it == hash.constEnd() ) {
            return QList<Nepomuk::Sync::ChangeLogRecord>();
        }

        return it->prop.values( propUri );
    }

}

//TODO: Add completed signal
void Nepomuk::SyncRequest::merge()
{
    kDebug();
    const Types::Property mergeableProperty( Nepomuk::Vocabulary::backupsync::mergeable() );
    //
    // Get own changeLog
    //
    kDebug() << "minDateTime : " << d->m_minDateTime;
    Sync::ChangeLog ownLog = LogStorage::instance()->getChangeLog( d->m_minDateTime );
    kDebug() << "own Log : " << ownLog.size();

    // Get our and their hash
    // ownHash = current local hash from system's ChangeLog
    // theirHash = derived from external ChangeLog
    Sync::ResourceLogMap ownHash = Sync::createResourceLogMap( ownLog );
    Sync::ResourceLogMap & theirHash = d->m_hash;
    kDebug() << "own Hash : " << ownHash.size();
    kDebug() << "their hash : " << theirHash.size();

    QHashIterator<QUrl, Sync::ResourceLog> it( theirHash );
    while( it.hasNext() ) {
        it.next();

        // Check for resource deletions
        if( d->handleResourceDeletion( it.key() ) )
            continue;

        const QUrl & resUri = it.key();
        const Sync::ResourceLog & resLog = it.value();

        kDebug() << "Resolving " << resUri;

        const QList<QUrl> & properties = resLog.prop.uniqueKeys();
        foreach( const QUrl & propUri, properties ) {
            kDebug() << propUri;

            if( !isMergeable( propUri, d->m_model ) ) {
                kDebug() << propUri << " is non Mergeable - IGNORING";
                continue;
            }

            Nepomuk::Types::Property prop( propUri );
            int cardinality = prop.maxCardinality();

            QList<Sync::ChangeLogRecord> theirRecords = resLog.prop.values( propUri );
            QList<Sync::ChangeLogRecord> ownRecords = getRecords( ownHash, resUri, propUri );
            //kDebug() << "own Records : " << ownRecords.size();

            // This case shouldn't ever happen, but just to be sure
            if( theirRecords.empty() )
                continue;

            if( cardinality == 1 ) {
                resolveSingleCardinality( theirRecords, ownRecords );
            }
            else {
                resolveMultipleCardinality( theirRecords, ownRecords );
            }
        }

        //if( !rs.propHash.isEmpty() )
        //    d->m_jobs.append( rs );
    }
    //theirHash.clear();
    //kDebug() << "Done with merge resolution : " << d->m_jobs.size();

    //processJobs();
}


namespace {

    Nepomuk::Sync::ChangeLogRecord maxRecord( const QList<Nepomuk::Sync::ChangeLogRecord> & records ) {
        QList<Nepomuk::Sync::ChangeLogRecord>::const_iterator it = std::max_element( records.begin(), records.end() );
        if( it != records.constEnd() )
            return *it;
        return Nepomuk::Sync::ChangeLogRecord();
    }
}


void Nepomuk::SyncRequest::resolveSingleCardinality(const QList< Nepomuk::Sync::ChangeLogRecord >& theirRecords, const QList< Nepomuk::Sync::ChangeLogRecord >& ownRecords)
{
    kDebug() << "O: " << ownRecords.size() << " " << "T:" << theirRecords.size();

    //Find max on the basis of time stamp
    Sync::ChangeLogRecord theirMax = maxRecord( theirRecords );
    Sync::ChangeLogRecord ownMax = maxRecord( ownRecords );
    kDebug() << "TheirMax : "<< theirMax.toString();
    kDebug() << "OwnMax " << ownMax.toString();

    if( theirMax > ownMax ) {
        Soprano::Statement statement( theirMax.st().subject(), theirMax.st().predicate(),
                                      Soprano::Node(), Soprano::Node() );

        if( theirMax.added() ) {
            Soprano::Node object = theirMax.st().object();
            kDebug() << "Resolved - Adding " << object;

            if( !d->m_model->containsAnyStatement( statement ) ) {
                statement.setObject( object );
                statement.setContext( d->m_graphWrapper.currentGraph() );
                d->m_model->addStatement( statement );
            }
        }
        else {
            kDebug() << "Resolved - Removing";
            d->m_model->removeAllStatements( statement );
        }
    }
}

namespace {

    struct MergeData {
        bool added;
        QDateTime dateTime;

        MergeData( bool add, const QDateTime & dt )
            : added( add ),
              dateTime( dt )
        {}
    };


}

void Nepomuk::SyncRequest::resolveMultipleCardinality( const QList<Nepomuk::Sync::ChangeLogRecord>& theirRecords, const QList<Nepomuk::Sync::ChangeLogRecord>& ownRecords)
{
    kDebug() << "MULTIPLE";
    kDebug() << "O: " << ownRecords.size() << " " << "T:" << theirRecords.size();

    const Soprano::Statement& reference = theirRecords.first().st();
    Soprano::Statement baseStatement( reference.subject(), reference.predicate(), Soprano::Node(), Soprano::Node() );

    //
    // Merge both record lists
    //
    //TODO: Optimize merging - use merge sort or something equivilant
    QList<Sync::ChangeLogRecord> records = ownRecords;
    records << theirRecords;
    qSort( records );

    QHash<Soprano::Node, MergeData> hash;
    foreach( const Sync::ChangeLogRecord rec, records ) {
        Soprano::Node object = rec.st().object();
        QHash<Soprano::Node, MergeData>::const_iterator it = hash.constFind( object );
        if( it == hash.constEnd() ) {
            hash.insert( object, MergeData( rec.added(), rec.dateTime() ) );
        }
        else {
            // +ve after -ve
            if( rec.added() == true && it.value().added == false ) {
                hash.remove( object );
                hash.insert( object, MergeData( rec.added(), rec.dateTime() ) );
            }
            // -ve after +ve
            else if( rec.added() == false && it.value().added == true ) {
                hash.remove( object );
            }
            // +ve after +ve
            // -ve after -ve
            //    Do nothing
        }
    }

    //
    // Do the actual merging
    //
    QHashIterator<Soprano::Node, MergeData> it( hash );
    while( it.hasNext() ) {
        it.next();

        Soprano::Statement st( baseStatement );
        st.setObject( it.key() );

        MergeData data = it.value();
        if( data.added == true ) {
            if( !d->m_model->containsAnyStatement( st ) ) {
                st.setContext( d->m_graphWrapper.currentGraph() );
                d->m_model->addStatement( st );
                kDebug() << "adding - " << st;
            }
        }
        else {
            kDebug() << "removing " << st;
            d->m_model->removeAllStatements( st );
        }
    }

    emit multipleMerge( baseStatement.subject().uri().toString(),
                        baseStatement.predicate().uri().toString() );
}


bool Nepomuk::SyncRequest::Private::handleResourceDeletion(const QUrl& resUri)
{
    Sync::ResourceLog & log = m_hash[ resUri ];
    const QUrl& rdfTypeProp = Soprano::Vocabulary::RDF::type();

    QList<Sync::ChangeLogRecord> records = log.prop.values( rdfTypeProp );
    if( records.empty() )
        return false;

    //
    // Check if rdf:type is being removed
    //
    bool removed = false;
    foreach( const Sync::ChangeLogRecord & r, records ) {
        if( !r.added() ) {
            removed = true;
            break;
        }
    }
    if( !removed )
        return false;

    // If removed, remove all records and delete the resource
    m_hash.remove( resUri );
    Resource res( resUri );
    res.remove();
    return true;
}


int Nepomuk::SyncRequest::size() const
{
    return d->m_logFile.size();
}

#include "syncrequest.moc"
