/* This file is part of the Nepomuk Project
   Copyright (c) 2010 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef _NEPOMUK_DATA_MAINTENANCE_SERVICE_H_
#define _NEPOMUK_DATA_MAINTENANCE_SERVICE_H_

#include <Nepomuk/Service>

#include <QtCore/QVariant>
#include <QtCore/QUrl>
#include <QtCore/QList>
#include <QtCore/QPair>
#include <QtCore/QRegExp>
#include <QtCore/QDateTime>

#include "usageevent.h"

class KUrl;

namespace Nepomuk {

    class Resource;

    class DataMaintenanceService : public Service
    {
        Q_OBJECT
        Q_CLASSINFO( "D-Bus Interface", "org.kde.nepomuk.DataMaintenanceService" )

    public:
        DataMaintenanceService( QObject* parent, const QVariantList& );
        ~DataMaintenanceService();

    public Q_SLOTS:
        /**
         * Called by clients to notify that they opened a resource for viewing or editing.
         * A desktop event resource will be created.
         *
         *
         */
        Q_SCRIPTABLE void resourceOpened( const QString& uri, const QString& service );
        Q_SCRIPTABLE void resourceOpenedOneShot( const QString& uri, const QString& service );
        Q_SCRIPTABLE void resourceSaved( const QString& uri, const QString& service );
        Q_SCRIPTABLE void resourceClosed( const QString& uri, const QString& service );

        /**
         * Calculate the score for the resource identified by \p uri. The score will not
         * be saved.
         */
        Q_SCRIPTABLE double scoreForResource( const QString& uri );

        /**
         * In contrast to scoreForResource this will actually change the resource score
         * in the database.
         */
        Q_SCRIPTABLE void updateResourceScore( const QString& uri );

        /**
         * Update all file scores. This process takes a while, thus the method returns
         * immediately while the update is processed asynchronously.
         */
        Q_SCRIPTABLE void updateFileScores();

    private:
        /**
         * Calculates the score for resource \p res.
         * \param res The resource to re-calculate the score for
         * \param now The current time, used as a parameter to make sure the same time
         * is used for all score recalculations in a loop.
         */
        double calculateScore( const Nepomuk::Resource& res, const QDateTime& now );

        /**
         * recreates m_filenamePatternWeightRxCache
         */
        void rebuildFilenamePatternWeightRxCache();

        /**
         * Lookup the weight for a specific NIE type in the type weight config. Falls back
         * to 1.0 for types that are not in the config.
         */
        double lookupTypeWeight( const KUrl& type ) const;

        /**
         * Lookup the weight for a filename based on the patterns in the weight config.
         * The first matching pattern is used to read the weight.
         *
         * m_filenamePatternWeightRxCache has to be created via rebuildFilenamePatternWeightRxCache()
         * for this method to work properly.
         */
        double lookupFilenamePatternWeight( const QString& filename ) const;

        struct UsageEventContainer {
            Resource resource;
            QString service;
            QDateTime timestamp;
            UsageEvent event;
        };
        QList<UsageEventContainer> m_pendingEvents;

        UsageEventContainer createUsageEvent( const QString& uri, const QString& service ) const;
        QList<UsageEventContainer>::iterator findPendingUsageEvent( const QString& uri, const QString& service, bool ignoreTimestamp );

        /**
         * a simple cache to avoid recreating of QRexExp objects on each call
         * or lookupFilenamePatternWeight(). Built in rebuildFilenamePatternWeightRxCache()
         */
        QList<QPair<QRegExp, double> > m_filenamePatternWeightRxCache;

        class ScoringThread;
        ScoringThread* m_scoringThread;
    };
}

#endif
