/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>
Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourceitemdelegate.h"
#include "resourceeditwidget.h"
#include "resourceeditwidgetwrapper.h"

#include <QWidget>
#include <QAbstractItemModel>
#include <QAbstractItemView>
#include <QStyleOptionViewItem>
#include <QStyledItemDelegate>
#include <QModelIndex>
#include <QEvent>
#include <QLocale>
#include <QPainter>
#include <QStyleOptionViewItemV4>
#include <QPixmap>
#include <QSize>
#include <QRect>
#include <QTextOption>
#include <QFont>

#include <KDebug>
#include <KIcon>
#include <KIconLoader>

Q_DECLARE_METATYPE(Nepomuk::Resource)
Q_DECLARE_METATYPE(KIcon) //for some reason, the compiler didn't pick it up

class Nepomuk::ResourceItemDelegate::Private {
public:
    bool hasDescription;
};

Nepomuk::ResourceItemDelegate::ResourceItemDelegate(QAbstractItemView * parent) :
        KExtendableItemDelegate(parent),
        d(new Private())
{
}

Nepomuk::ResourceItemDelegate::~ResourceItemDelegate()
{
    delete d;
}

QSize Nepomuk::ResourceItemDelegate::sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const
{
    QStyleOptionViewItemV4 itemOption(option);

    QSize size(KExtendableItemDelegate::sizeHint(itemOption, index));
    if (isExtended(index)) {
        //size.rheight() += 5;
    } else {
        size.rheight() = itemOption.fontMetrics.height() * (index.data(ResourceDescriptionRole).toString().isEmpty() ? 1 : 2);//if we have a description, we add another line
    }
    return size;
}

void Nepomuk::ResourceItemDelegate::paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const
{
    //KExtendableItemDelegate::paint(painter, option, index);
    if(isExtended(index)) {
        KExtendableItemDelegate::paint(painter, option, index);
    }
    else { //custom painting, if not extended
        //QStyledItemDelegate::paint(painter, option, index);
        QStyleOptionViewItemV4 itemOption(option);
        int iconSize = itemOption.decorationSize.width();

        //FIXME: icon doesn't get drawn
        QPixmap iconPixmap = index.data(Qt::DecorationRole).value<KIcon>().pixmap(iconSize, iconSize);

        //This is the layout:
        // |icon| Label                                 Type
        // |icon| Descr.....................................

        //FIXME: draw in a rect, align vertically to center
        QRect iconRect(itemOption.rect.x(), itemOption.rect.y(), iconSize + 11, iconSize + 11 );

        int fontHeight = itemOption.fontMetrics.height();
        int textWidth = itemOption.rect.width() - iconRect.width();
        int textX = itemOption.rect.x() + iconRect.width();
        int texY = itemOption.rect.y();

        QRect labelRect = QRect(textX,
                                texY,
                                textWidth / 2,
                                fontHeight);

        QRect typeRect = QRect(textX + textWidth / 2,
                               texY,
                               textWidth / 2 - 11,
                               fontHeight);

        QRect descrRect = QRect(textX + 11, //indent it a bit
                                texY + fontHeight,
                                textWidth,
                                fontHeight * 2);

        //draw icon
        painter->drawPixmap(iconRect, iconPixmap);

        //draw label
        painter->drawText( labelRect, Qt::AlignLeft, index.data(Qt::DisplayRole).toString() );

        //draw type
        painter->save();
        QFont font(itemOption.font);
        font.setItalic(true);
        painter->setFont(font);
        painter->drawText( typeRect, Qt::AlignRight, index.data(ResourceTypeRole).toString() );;
        painter->restore();

        //draw description
        painter->save();
        font = itemOption.font;
        font.setWeight(QFont::Light);
        painter->setFont(font);;
        QString descrText = index.data(ResourceDescriptionRole).toString();
        if(!descrText.isEmpty()) {
            painter->drawText( descrRect, Qt::AlignLeft, descrText );
        }

        painter->restore();
    }

}

QWidget *Nepomuk::ResourceItemDelegate::createEditor(QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index) const
{
    Q_UNUSED(option);

    //kDebug() << "Creating editor";
    Resource resource =  index.model()->data(index, Qt::EditRole).value<Resource>();

    ResourceEditWidgetWrapper *wrapperWidget = new ResourceEditWidgetWrapper(parent);

    ResourceEditWidget *editWidget = new ResourceEditWidget(wrapperWidget, resource);
    editWidget->setResource(resource);//TODO: Remove this later. It is just to test that setting the resource twice works.
    connect(editWidget, SIGNAL(geometryChanged()), parent, SLOT(refreshGeometries()));
    connect(editWidget, SIGNAL(geometryChanged()), this, SLOT(emitGeometryChanged()));

    wrapperWidget->setResourceEditWidget(editWidget);

    return wrapperWidget;
}

void Nepomuk::ResourceItemDelegate::updateEditorGeometry(QWidget *editor,
                                        const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    KExtendableItemDelegate::updateEditorGeometry(editor, option, index);
    editor->setGeometry(option.rect);
}

void Nepomuk::ResourceItemDelegate::updateExtenderGeometry(QWidget *extender, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    KExtendableItemDelegate::updateExtenderGeometry(extender, option, index);
    extender->setGeometry(option.rect);
    emit const_cast<Nepomuk::ResourceItemDelegate*>(this)->sizeHintChanged(index);//is this a bad idea?
}

bool Nepomuk::ResourceItemDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option, const QModelIndex &index)
{
    bool retVal = KExtendableItemDelegate::editorEvent(event, model, option, index);

    emitGeometryChanged();

    return retVal;
}

void Nepomuk::ResourceItemDelegate::emitGeometryChanged()
{
    QAbstractItemView *view = qobject_cast<QAbstractItemView*>(parent());
    if(view) {
        for(int i=0; i<view->model()->rowCount(); i++) {
            emit sizeHintChanged(view->model()->index(i, 0));
        }
    }
    //kDebug() << "*** Emitting geometry change";
}
