/***************************************************************************
 *   Copyright 2008 by Thomas Schildknecht <danakil.kde@gmail.com>         *
 *   Copyright 2010 by Stefan Majewsky <majewsky@gmx.net>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 *                                                                         *
 ***************************************************************************/

//TODO idées : dans tasks dataengine : sources() qui retourne la liste d'id
// sous forme de WId

#include "activewindowcontrol.h"

#include <QGraphicsLinearLayout>
#include <QGraphicsSceneWheelEvent>
#include <QListWidget>
#include <KConfigDialog>

ActiveWindowControl::ActiveWindowControl(QObject *parent, const QVariantList &args) :
        Plasma::Applet(parent, args),
        m_isVerticalLayout(false)
{
    setHasConfigurationInterface(true);
}

void ActiveWindowControl::init()
{
    KConfigGroup cg = config();
    //create window manager
    Plasma::DataEngine* tasksEngine = dataEngine("tasks");
    if (!tasksEngine) {
        setFailedToLaunch(true, i18n("Could not load tasks dataEngine."));
        return;
    }
    m_windowManager = new Awc::WindowManager(tasksEngine, this);
    connect(this, SIGNAL(scrollWheelOnIcon(int)), m_windowManager, SLOT(cycleToAdjacentWindow(int)));
    //read configuration
    m_buttonTypes = cg.readEntry("idActionsSelected", m_buttonTypes);
    m_isVerticalLayout = cg.readEntry("verticalLayout", m_isVerticalLayout);
    const int cyclingStrategy = cg.readEntry("cyclingStrategy", (int) m_windowManager->cyclingStrategy());
    m_windowManager->setCyclingStrategy((Awc::CyclingStrategy) cyclingStrategy);
    m_isOnPanel = (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical);
    m_windowManager->setReactivateActiveWindowAfterOperations(!m_isOnPanel);
    //create default configuration on first run
    if (m_buttonTypes.isEmpty()) {
        m_buttonTypes << Awc::MinimizeButton << Awc::MaximizeButton << Awc::SeparatorButton << Awc::CloseButton;
        cg.writeEntry("idActionsSelected", m_buttonTypes);
        emit configNeedsSaving();
    }
    //build layout
    m_layout = new QGraphicsLinearLayout;
    m_layout->setContentsMargins(0, 0, 0, 0);
    m_layout->setSpacing(0);
    m_layout->setOrientation(m_isVerticalLayout ? Qt::Vertical : Qt::Horizontal);
    setLayout(m_layout);
    rebuildButtons();
}

void ActiveWindowControl::createConfigurationInterface(KConfigDialog* parent)
{
    QWidget* widget = new QWidget(parent);
    m_uiConfig.setupUi(widget);
    parent->addPage(widget, i18nc("caption of configuration page", "General"), "configure");
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));

    connect(m_uiConfig.winactionselector, SIGNAL(added(QListWidgetItem*)), this, SLOT(slotActionAdded(QListWidgetItem *)));
    connect(m_uiConfig.winactionselector, SIGNAL(removed(QListWidgetItem*)), this, SLOT(slotActionRemoved(QListWidgetItem*)));

    //load configuration into config UI
    Awc::CyclingStrategy strategy = m_windowManager->cyclingStrategy();
    m_uiConfig.groupBoxCycling->setChecked(strategy & Awc::CyclingEnabled);
    m_uiConfig.checkBoxCycleAllDesktops->setChecked(strategy & Awc::CycleOnAllDesktops);
    m_uiConfig.checkBoxCycleMinimized->setChecked(strategy & Awc::IncludeMinimizedWindows);
    m_uiConfig.checkBoxVerticalLayout->setVisible(!m_isOnPanel);
    m_uiConfig.checkBoxVerticalLayout->setChecked(m_isVerticalLayout);
    //add the selected actions
    foreach (int idAction, m_buttonTypes) {
        QListWidgetItem* item = buildItem((Awc::ButtonType) idAction);
        m_uiConfig.winactionselector->selectedListWidget()->addItem(item);
    }
    //add the available actions (the separator is always added because it needs to be available for multiple insertion)
    for (int idAction = 0; idAction < Awc::ButtonTypeCount; ++idAction) {
        if (idAction == 0 || !m_buttonTypes.contains(idAction)) {
            QListWidgetItem* item = buildItem((Awc::ButtonType) idAction);
            m_uiConfig.winactionselector->availableListWidget()->addItem(item);
        }
    }
}

QListWidgetItem* ActiveWindowControl::buildItem(Awc::ButtonType buttonType) const
{
    Awc::ButtonMetadata metadata(buttonType);
    QListWidgetItem* item = new QListWidgetItem(metadata.renderedIcon(), metadata.m_actionText);
    item->setData(Qt::UserRole + 1, buttonType);
    return item;
}

void ActiveWindowControl::slotActionAdded(QListWidgetItem* item)
{
    if (item->data(Qt::UserRole + 1) == Awc::SeparatorButton) {
        QListWidgetItem* item = buildItem(Awc::SeparatorButton);
        m_uiConfig.winactionselector->availableListWidget()->addItem(item);
    }
}

void ActiveWindowControl::slotActionRemoved(QListWidgetItem* item)
{
    if (item->data(Qt::UserRole + 1) == Awc::SeparatorButton) {
        delete item;
    }
}

void ActiveWindowControl::configAccepted()
{
    KConfigGroup cg = config();

    // Save the KActionSelector selected list
    m_buttonTypes.clear();
    for (int i = 0; i < m_uiConfig.winactionselector->selectedListWidget()->count(); i++) {
        const int value = m_uiConfig.winactionselector->selectedListWidget()->item(i)->data(Qt::UserRole + 1).toInt();
        m_buttonTypes.append(value);
    }
    // kDebug()<<"############################# sauvegarde idlist "<<actionIdList;

    // Get the others settings
    m_isVerticalLayout = m_uiConfig.checkBoxVerticalLayout->isChecked();
    m_layout->setOrientation(m_isVerticalLayout ? Qt::Vertical : Qt::Horizontal);
    Awc::CyclingStrategy strategy;
    if (m_uiConfig.groupBoxCycling->isChecked()) {
        strategy |= Awc::CyclingEnabled;
    }
    if (m_uiConfig.checkBoxCycleAllDesktops->isChecked()) {
        strategy |= Awc::CycleOnAllDesktops;
    }
    if (m_uiConfig.checkBoxCycleMinimized->isChecked()) {
        strategy |= Awc::IncludeMinimizedWindows;
    }
    m_windowManager->setCyclingStrategy(strategy);

    cg.writeEntry("idActionsSelected", m_buttonTypes);
    cg.writeEntry("verticalLayout", m_isVerticalLayout);
    cg.writeEntry("cyclingStrategy", (int) strategy);

    emit configNeedsSaving();
    rebuildButtons();
}

void ActiveWindowControl::rebuildButtons()
{
    //remove all buttons
    qDeleteAll(m_buttons);
    m_buttons.clear();
    //build button list from scratch
    foreach (int buttonType, m_buttonTypes) {
        Awc::Button* button = new Awc::Button((Awc::ButtonType) buttonType, m_windowManager, this);
        m_buttons << button;
        m_layout->addItem(button);
    }
    //done
    updateConstraints();
}

void ActiveWindowControl::wheelEvent(QGraphicsSceneWheelEvent* event)
{
    event->accept();
    emit scrollWheelOnIcon(event->delta() >= 0 ? 1 : -1);
}

K_EXPORT_PLASMA_APPLET(activewindowcontrol, ActiveWindowControl)
#include "activewindowcontrol.moc"
