/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "annotatedmarblewidget.h"

#include <QtGui/QLinearGradient>
#include <QtCore/QMutexLocker>

#include <kdebug.h>

#include <Plasma/PaintUtils>

AnnotatedMarbleWidget::AnnotatedMarbleWidget(QWidget *parent) :
	Marble::MarbleWidget(parent), m_recenterOnLocationChange(true)
{
	// nothing to do
}

AnnotatedMarbleWidget::~AnnotatedMarbleWidget()
{
	// nothing to do
}

void AnnotatedMarbleWidget::customPaint(Marble::GeoPainter *painter)
{
	painter->setPen(QColor(0, 0, 0));

	QLinearGradient linearGrad(0, 90, 60, 0);
	linearGrad.setColorAt(0, QColor(255, 255, 148, 140));
	linearGrad.setColorAt(1, QColor(255, 255, 98, 140));
	QBrush gradientBrush(linearGrad);
	painter->setBrush(linearGrad);

	QMutexLocker locker(&m_annotationsMutex);
	foreach(const Annotation &annotation, m_annotations)
	{
		painter->drawAnnotation( annotation.point, annotation.text, QSize(120, 80), 10, 30, 15, 15 );
	}
}

void AnnotatedMarbleWidget::propertiesChanged(const QString &id,
		const QVariantMap &properties)
{
	qreal longitude = properties["longitude"].toDouble();
	qreal latitude = properties["latitude"].toDouble();
	if (properties["location_known"].toBool())
	{
		Annotation annotation;
		annotation.point = Marble::GeoDataPoint(longitude, latitude, 0.0,
				Marble::GeoDataPoint::Degree);
		annotation.text = properties["name"].toString();

		if (m_recenterOnLocationChange && !m_annotations.contains(id))
		{
			// First time we know the location of this contact, move map center to it
			setCenterLatitude(latitude);
			setCenterLongitude(longitude);
		}
		/** @todo: check whether location has changed and update center according to preference */

		m_annotations[id] = annotation;
	}
	else
	{
		QMutexLocker locker(&m_annotationsMutex);
		m_annotations.remove(id);
	}

	/** @todo: any update needed? */
	// updateChangedMap();
}

void AnnotatedMarbleWidget::resizeEvent ( QResizeEvent * event )
{
	Q_UNUSED(event);

	QSizeF widgetSize = size();
	QPainterPath roundPath = Plasma::PaintUtils::roundedRectangle(QRectF(5,5,widgetSize.width()-10,widgetSize.height()-10),12);
	setMask(roundPath.toFillPolygon().toPolygon());
	MarbleWidget::resizeEvent(event);
}

void AnnotatedMarbleWidget::setRecenterOnLocationChange(bool enabled)
{
	m_recenterOnLocationChange = enabled;
}
