/*
 *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser/Library General Public License version 2,
 *   or (at your option) any later version, as published by the Free
 *   Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser/Library General Public License for more details
 *
 *   You should have received a copy of the GNU Lesser/Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "embedded_tasklauncher.h"
#include <KDebug>
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QDir>

#include <KRun>

#include <plasma/containment.h>
#include <plasma/panelsvg.h>
#include <plasma/theme.h>
#include <plasma/animator.h>

#include <climits>

EmbeddedTasklauncherApplet::TaskData::TaskData(QString desktopFileLocation, StatesIcon * icon)
    : iconWidget(NULL), desktopFile(desktopFileLocation)
{
    KConfigGroup gr = desktopFile.group("PlasmaEmbeddedExtras");
    className       = gr.readEntry("taskClassName", QString());
    classClass      = gr.readEntry("taskClassClass", QString());

    iconWidget = icon;

    kDebug() << "Running"  << gr.readEntry("runningIcon",  desktopFile.readIcon());
    kDebug() << "Offline"  << gr.readEntry("offlineIcon",  desktopFile.readIcon());
    kDebug() << "Starting" << gr.readEntry("startingIcon", desktopFile.readIcon());
    kDebug() << "Active"   << gr.readEntry("activeIcon",   desktopFile.readIcon());

    iconWidget->registerState(StatesIcon::Running,  KIcon(gr.readEntry("runningIcon",  desktopFile.readIcon())));
    iconWidget->registerState(StatesIcon::Offline,  KIcon(gr.readEntry("offlineIcon",  desktopFile.readIcon())));
    iconWidget->registerState(StatesIcon::Starting, KIcon(gr.readEntry("startingIcon", desktopFile.readIcon())));
    iconWidget->registerState(StatesIcon::Active,   KIcon(gr.readEntry("activeIcon",   desktopFile.readIcon())));

    iconWidget->setBackgroundSvg("embedded/tasklauncher-icon-background");
}

EmbeddedTasklauncherApplet::EmbeddedTasklauncherApplet(QObject * parent, const QVariantList & args):
    Plasma::Applet(parent, args)
{
    setHasConfigurationInterface(false);
    // setBackgroundHints(NoBackground);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    resize(500, 58);
}

EmbeddedTasklauncherApplet::~EmbeddedTasklauncherApplet()
{
    foreach (TaskData * taskData, m_managedTasks) {
        delete taskData->iconWidget;
        delete taskData;
    }
}

void EmbeddedTasklauncherApplet::init()
{
    setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
    setMaximumSize(INT_MAX, INT_MAX);

    connect(TaskManager::TaskManager::self(), SIGNAL(taskAdded(TaskPtr)),
            this, SLOT(taskAdded(TaskPtr)));
    connect(TaskManager::TaskManager::self(), SIGNAL(taskRemoved(TaskPtr)),
            this, SLOT(taskRemoved(TaskPtr)));
    connect(TaskManager::TaskManager::self(), SIGNAL(startupAdded(StartupPtr)),
            this, SLOT(startupAdded(StartupPtr)));
    connect(TaskManager::TaskManager::self(), SIGNAL(startupRemoved(StartupPtr)),
            this, SLOT(startupRemoved(StartupPtr)));

    m_layout = new QGraphicsLinearLayout(this);
    m_layout->setContentsMargins(0,0,0,0);
    m_layout->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));

    // load needed buttons
    QDir dir = QDir::home();
    dir.cd(".tasks");
    dir.setFilter(QDir::Files);

    QStringList filters;
    filters << "*.desktop";
    dir.setNameFilters(filters);

    StatesIcon * icon;
    foreach (QString file, dir.entryList()) {
        kDebug() << "Desktop file" << dir.absoluteFilePath(file);

        icon = new StatesIcon(this);
        m_managedTasks.append(new TaskData(dir.absoluteFilePath(file), icon));
        connect(
                icon, SIGNAL(clicked()),
                this, SLOT(iconClicked())
        );
        icon->setState(StatesIcon::Offline);

        m_layout->addItem(icon);
    }

    setLayout(m_layout);

    QMapIterator<WId,TaskPtr> iter(TaskManager::TaskManager::self()->tasks());
    while (iter.hasNext()) {
        iter.next();
        taskAdded(iter.value());
    }

}

void EmbeddedTasklauncherApplet::taskAdded(TaskPtr task)
{
    connect(
            task.constData(), SIGNAL(activated()),
            this, SLOT(taskActivated())
    );

    kDebug() << "Task data" << task->className() << task->classClass();
    foreach (TaskData * data, m_managedTasks) {
        kDebug() << "We need" << data->className << data->classClass;
        if (data->className != task->className() || data->classClass != task->classClass()) {
            continue;
        }

        data->task = task;
        if (task->isActive()) {
            data->iconWidget->setState(StatesIcon::Active);
        } else {
            data->iconWidget->setState(StatesIcon::Running);
        }
        break;
    }
}

void EmbeddedTasklauncherApplet::taskRemoved(TaskPtr task)
{
    disconnect(
            task.constData(), SIGNAL(activated()),
            this, SLOT(taskActivated())
    );

    foreach (TaskData * data, m_managedTasks) {
        if (data->className != task->className() || data->classClass != task->classClass()) {
            continue;
        }

        data->iconWidget->setState(StatesIcon::Offline);
        break;
    }
}

void EmbeddedTasklauncherApplet::startupAdded(StartupPtr task)
{
    foreach (TaskData * data, m_managedTasks) {
        kDebug() << data->desktopFile.readName() << task->text();
        if (data->desktopFile.readName() != task->text()) {
            continue;
        }

        if (data->iconWidget->state() == StatesIcon::Offline) {
            data->iconWidget->setState(StatesIcon::Starting);
        }
        break;
    }
}

void EmbeddedTasklauncherApplet::startupRemoved(StartupPtr task)
{
    foreach (TaskData * data, m_managedTasks) {
        kDebug() << data->desktopFile.readName() << task->text();
        if (data->desktopFile.readName() != task->text()) {
            continue;
        }

        if (data->iconWidget->state() == StatesIcon::Starting) {
            data->iconWidget->setState(StatesIcon::Offline);
        }
        break;
    }
}

void EmbeddedTasklauncherApplet::taskActivated()
{
    Task * task = (Task *)sender();
    foreach (TaskData * data, m_managedTasks) {
        if (data->task != task) {
            if (data->iconWidget->state() == StatesIcon::Active) {
                data->iconWidget->setState(StatesIcon::Running);
            }
            continue;
        }

        data->iconWidget->setState(StatesIcon::Active);
        break;
    }
    update();
}

void EmbeddedTasklauncherApplet::constraintsEvent(Plasma::Constraints constraints)
{
    Q_UNUSED(constraints);
}

void EmbeddedTasklauncherApplet::iconClicked()
{
    kDebug() << "Icon clicked";
    StatesIcon * icon = (StatesIcon *)sender();
    foreach (TaskData * data, m_managedTasks) {
        kDebug() << (void *)data->iconWidget << (void *)icon << "testing";
        if (data->iconWidget != icon) {
            continue;
        }

        kDebug() << "Match";
        switch (icon->state()) {
            case StatesIcon::Offline:
                new KRun(KUrl(data->desktopFile.fileName()), 0);
                break;
            case StatesIcon::Running:
                data->task->activate();
                break;
            default:
                break;
        }

        break;
    }
}

#include <embedded_tasklauncher.moc>
