/*
 *   Copyright (C) 2008 Schutz Sacha <istdasklar@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2,
 *   or (at your option) any later version, as published by the Free
 *   Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "kfortunoid.h"

#include <QPainter>
#include <QProcess>
#include <QFile>
#include <QDebug>
#include <QGraphicsLinearLayout>
#include <QLabel>
#include <QTimer>
#include <QTextCodec>

#include <Plasma/Animator>
#include <Plasma/Label>
#include <Plasma/ToolButton>
#include <Plasma/Frame>

#include <KConfigDialog>
#include <KStandardDirs>
#include <KConfigGroup>

#include "fortunoidsettings.h"

KFortunoid:: KFortunoid(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_currentQuote(0),
    m_panelFont(KGlobalSettings::smallestReadableFont()),
    m_pathFortune()
{
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    m_delay = 2;
    m_arguments=QString();
    m_font=QFont();
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
    m_color = Qt::white;
    m_labelQuote = new  Plasma::Label;
    m_labelQuote->nativeWidget()->setTextInteractionFlags(Qt::TextSelectableByMouse);

    int minWidth = 300;
    setMinimumWidth(minWidth);
    // calculate minimum number of characters needed to fit fortune text
    // in panel, based on minWidth and assuming height of two lines.
    // Then we pass this to fortune as a parameter
    QFontMetrics fm(m_panelFont);
    int xWidth = fm.width('X');
    int optimalNumberOfChars = (minWidth * 2)/xWidth;

    // this should keep fortunes fairly short
    m_panelArguments = QString("-s -n %1").arg(optimalNumberOfChars);

    m_timer = new QTimer(this);
    connect(m_timer,SIGNAL(timeout()), this, SLOT(slotNextQuote()));

    //TODO: Set history size on config screen
    m_history = 10;
}

KFortunoid::~ KFortunoid()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
    }
}

void KFortunoid::init()
{
    QProcess *find = new QProcess();
    find->start("whereis -b fortune");
    find->waitForFinished();

    QString result = find->readAll();
    result.remove("fortune: ");
    result.remove('\n');

    QStringList paths = result.split(' ');

    foreach(const QString& path, paths) {
        if(QFile::exists(path)) {
            m_pathFortune = path;
            break;
        }
    }

    if (m_pathFortune.isEmpty()) {
        setFailedToLaunch(true, i18n("Cannot find ") + m_pathFortune);
        return;
    }

    m_controlFrame = new Plasma::Frame( this );
    m_controlFrame->setZValue( 10 );

    m_backButton = new Plasma::ToolButton(m_controlFrame);
    m_backButton->setImage("widgets/arrows", "left-arrow");
    m_backButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_backButton->setMaximumSize(IconSize(KIconLoader::MainToolbar), IconSize(KIconLoader::MainToolbar));
    connect(m_backButton, SIGNAL(clicked()), this, SLOT(slotPrevQuote()));

    m_nextButton = new Plasma::ToolButton(m_controlFrame);
    m_nextButton->setImage("widgets/arrows", "right-arrow");
    m_nextButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_nextButton->setMaximumSize(IconSize(KIconLoader::MainToolbar), IconSize(KIconLoader::MainToolbar));
    connect(m_nextButton, SIGNAL(clicked()), this, SLOT(slotNextQuote()));

    QGraphicsLinearLayout *controlLayout = new QGraphicsLinearLayout();
    controlLayout->addItem(m_backButton);
    controlLayout->addItem(m_nextButton);
    m_controlFrame->setLayout(controlLayout);
    controlLayout->activate();

    m_controlFrame->setFrameShadow(Plasma::Frame::Raised);
    m_controlFrame->hide();

    m_labelQuote->nativeWidget()->setAlignment(Qt::AlignCenter);

    QGraphicsLinearLayout *mainLayout = new QGraphicsLinearLayout(this);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addItem(m_labelQuote);
    setLayout(mainLayout);

    //resize(400, 200);
    readData();
    refresh();
}

void KFortunoid::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED(option)
    Q_UNUSED(contentsRect)

    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    Plasma::Applet::paintInterface(p, option, contentsRect);
}

QString KFortunoid::getNewQuote()
{
    QProcess *fortune = new QProcess();

    QString args = m_arguments;

    if (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical)
        args.append(' ' + m_panelArguments);

    fortune->start(m_pathFortune + ' ' + args);
    fortune->waitForFinished();

    if (fortune->exitCode() != 0)
        return QString();

    return fortune->readAll();
}

void KFortunoid::slotPrevQuote()
{
    if (m_currentQuote < m_quotes.size() - 1) {
        ++m_currentQuote;
        m_labelQuote->setText("<pre style='white-space:pre-wrap; color:"+m_color.name()+";'>"+m_quotes.at(m_currentQuote)+"</pre>" );
    }
}

void KFortunoid::slotNextQuote()
{
    if (m_currentQuote == 0) {
        QString newQuote = getNewQuote();

        if (newQuote.isEmpty()) {
            newQuote = QString("No avaiable quote!");
            m_labelQuote->setText("<pre style='white-space:pre-wrap; color:"+m_color.name()+";'>"+newQuote+"</pre>");
            return;
        }

        m_quotes.prepend(newQuote);

    } else {
        --m_currentQuote;
    }

    m_labelQuote->setText("<pre style='white-space:pre-wrap; color:"+m_color.name()+";'>"+m_quotes.at(m_currentQuote)+"</pre>" );

    // Do some clean up
    while (m_quotes.size() > m_history)
        m_quotes.remove(m_quotes.size() - 1);
}

void KFortunoid::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & Plasma::FormFactorConstraint) {
        refresh();
    }

    if (constraints & Plasma::SizeConstraint) {
        int x = contentsRect().center().x() - (m_controlFrame->size().width() / 2);
        int y = contentsRect().bottom() - m_controlFrame->size().height() - 5;

        m_controlFrame->setPos(x, y);
    }
}

void KFortunoid::createConfigurationInterface(KConfigDialog *parent)
{
    m_settings = new FortunoidSettings;
    parent->addPage(m_settings, i18n("Fortunoid Settings"),
                    KStandardDirs::locate("icon", "oxygen/scalable/apps/accessories-text-editor.svgz"));

    readData();
    m_settings->spinBoxDelay->setValue(m_delay);
    m_settings->lineEditArguments->setText(m_arguments);
    m_settings->FontRequester->setFont(m_font);
    m_settings->lineEditPath->setText(m_pathFortune);
    m_settings->kcolorbutton->setColor(m_color);


    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
}

//========================================================================

void KFortunoid::configAccepted()
{
    writeData();
    emit configNeedsSaving();
    refresh();
}

//==================================================================================

void KFortunoid::refresh()
{
    if (m_timer->isActive())
        m_timer->stop();
    m_timer->start(m_delay*1000);
    m_labelQuote->nativeWidget()->setFont( (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical) ? m_panelFont : m_font);
    slotNextQuote();
    update();
}

//=======================================================================================

void KFortunoid::readData()
{
    KConfigGroup cg = config();

    m_arguments = cg.readEntry("arguments", QString());
    m_delay = cg.readEntry("delay", 20);
    m_font = cg.readEntry("font", QFont());
    QColor c(Qt::white);
    m_color.setNamedColor(cg.readEntry("color",c.name()));
}
//==========================================================================================
void KFortunoid::writeData()
{
    m_delay = m_settings->spinBoxDelay->value();
    m_arguments = m_settings->lineEditArguments->text();
    m_font = m_settings->FontRequester->font();
    m_color=m_settings->kcolorbutton->color();


    KConfigGroup cg = config();
    cg.writeEntry("arguments", m_arguments);
    cg.writeEntry("delay", m_delay);
    cg.writeEntry("font", m_font);

    cg.writeEntry("color",m_color.name());
}

void KFortunoid::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    if (m_controlFrame)
        m_controlFrame->show();

    Applet::hoverEnterEvent(event);
}

void KFortunoid::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    if (m_controlFrame)
        m_controlFrame->hide();

    Applet::hoverLeaveEvent( event );
}

#include "kfortunoid.moc"
