/*
 *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser/Library General Public License version 2,
 *   or (at your option) any later version, as published by the Free
 *   Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser/Library General Public License for more details
 *
 *   You should have received a copy of the GNU Lesser/Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "JavaApplet.h"

#include "qtjambi_core.h"

#include <QGraphicsWidget>
#include <jni.h>

JavaApplet::JavaApplet(QObject * parent, const QVariantList & args)
  : Plasma::Applet(parent, args)
{
  setHasConfigurationInterface(false);
}

JavaApplet::~JavaApplet()
{
}

void JavaApplet::init()
{
    // Make sure the JVM is loaded·
    qtjambi_initialize_vm();

    // QGraphicsWidget * widget = jambi_QGraphicsWidgetInstance("org.kde.plasma.applets.java.MainWidget");
    QGraphicsWidget * widget = jambi_QGraphicsWidgetInstance("org/kde/plasma/applets/java/MainWidget");
    if (!widget) {
        setFailedToLaunch("Widget failed to instantiate");
        return;
    }
    widget->setParentItem(this);
    widget->setGeometry(0, 0, 400, 400);
}

QSizeF JavaApplet::contentSizeHint() const
{
    return QSizeF(400, 400);
}

QGraphicsWidget * JavaApplet::jambi_QGraphicsWidgetInstance(const char * className, QObject * parent)
{
    // Get the pointer to the current JNI environment
    JNIEnv * env = qtjambi_current_environment();
    if (env == 0) {
        qWarning("Cannot get current JNI environment. Was the JVM loaded correctly?");

        // Make sure a possible exception is removed from the stack
        qtjambi_exception_check(env);
        return 0;
    }

    // Make a new frame for local JNI refs
    env->PushLocalFrame(100);

    // Get a reference to the class
    jclass clazz = qtjambi_find_class(env, className);
    if (clazz == 0) {
        qWarning("Failed to find class");

        qtjambi_exception_check(env);
        return 0;
    }

    // Make sure the class actually extends QWidget
    jclass qwidget_class = qtjambi_find_class(env, "com/trolltech/qt/gui/QGraphicsWidget");
    if (qwidget_class == 0 || !env->IsAssignableFrom(clazz, qwidget_class)) {
        qWarning("Failed to find QGraphicsWidget class or suggested class is not a QGraphicsWidget subclass");

        qtjambi_exception_check(env);
        return 0;
    }

    // Get the ID of the default QGraphicsWidget constructor
    jmethodID constructor_id = env->GetMethodID(clazz, "<init>", "()V");
    if (constructor_id == 0) {
        qWarning("Failed to find default constructor");

        qtjambi_exception_check(env);
        return 0;
    }

    // Convert parent to java object
    jobject java_parent = parent != 0 ? qtjambi_from_QObject(env, parent) : 0;
    if ((java_parent == 0) != (parent == 0)) {
        qWarning("Failed to convert parent to java");

        qtjambi_exception_check(env);
        return 0;
    }

    // Instantiate the QWidget
    jobject java_widget = env->NewObject(clazz, constructor_id, java_parent);
    if (java_widget == 0) {
        qWarning("Failed to instantiate object");

        qtjambi_exception_check(env);
        return 0;
    }

    // Make sure the widget isn't garbage collected
    java_widget = env->NewGlobalRef(java_widget);
    if (java_widget == 0) {
        qWarning("Failed to create global reference to object");

        qtjambi_exception_check(env);
        return 0;
    }

    // Clean up local refs
    env->PopLocalFrame(0);

    return qobject_cast<QGraphicsWidget *>(qtjambi_to_qobject(env, java_widget));
}
