/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "videocapturebin.h"
#include <KDebug>
#include "libqtgstreamer/qgstbin.h"
#include "libqtgstreamer/qgstelement.h"
#include "libqtgstreamer/qgstelementfactory.h"
#include "libqtgstreamer/qgstghostpad.h"
#include "libqtgstreamer/qgstcaps.h"
#include "libqtgstreamer/qgststructure.h"
#include "libqtgstreamer/interfaces/qgstcolorbalance.h"
#include "videoinputdevice.h"

using namespace QtGstreamer;

class VideoCaptureBinPrivate
{
    public:
        VideoInputDevicePtr videoDevice;
        QGstElementPtr videoDeviceCapsFilter;
        QGstElementPtr videoQueue;
        QSize frameSize;
        bool adjustTimeStamp;
        long long deltaTimeStamp;
};

bool VideoCaptureBin::bufferProbe(GstPad *pad,GstBuffer *buffer,VideoCaptureBin *videoBin)
{
    //kDebug() << "Buffer in ts=" << GST_BUFFER_TIMESTAMP(buffer);
    if (videoBin->d->adjustTimeStamp == true) {
        kDebug() << "Buffer in ts=" << GST_BUFFER_TIMESTAMP(buffer);
        videoBin->d->adjustTimeStamp = false;
        videoBin->d->deltaTimeStamp = GST_BUFFER_TIMESTAMP(buffer);
        GstEvent * newSegmentEvent = gst_event_new_new_segment(FALSE,1,GST_FORMAT_TIME,0, -1, 0);
        QGstPadPtr sinkPad;
        sinkPad = videoBin->d->videoQueue->getStaticPad("sink");
        gst_pad_send_event(GST_PAD(sinkPad->peekNativeObject()),newSegmentEvent);
        GST_BUFFER_FLAG_SET(buffer,GST_BUFFER_FLAG_DISCONT);
        //kDebug() << "Buffer out ts=" << GST_BUFFER_TIMESTAMP(buffer);
    }
    GST_BUFFER_TIMESTAMP(buffer) = GST_BUFFER_TIMESTAMP(buffer) - videoBin->d->deltaTimeStamp;
    //kDebug() << "Buffer out ts=" << GST_BUFFER_TIMESTAMP(buffer);
    return true;
}

void VideoCaptureBin::newTimeStamp()
{
    QGstPadPtr pad = getStaticPad("src");
    gst_pad_add_buffer_probe(GST_PAD(pad->peekNativeObject()),G_CALLBACK(&bufferProbe),this);
    d->adjustTimeStamp = true;
}

VideoCaptureBinPtr VideoCaptureBin::createVideoCaptureBin()
{
    QSize frameSize(320,240);
    QString driver="autovideosrc";
    QString devicePath=QString();
    VideoInputDevicePtr videoDevice = VideoInputDevice::newDevice(driver,devicePath);
    QGstElementPtr capsFilter = QGstElementFactory::make("capsfilter");
    
    if ( !videoDevice || !capsFilter) {
        return VideoCaptureBinPtr();
    }

    VideoCaptureBin *videoCaptureBin = new VideoCaptureBin();
    videoCaptureBin->d->adjustTimeStamp=false;
    videoCaptureBin->d->deltaTimeStamp=0;
    videoCaptureBin->d->videoDevice = videoDevice;
    videoCaptureBin->d->videoDeviceCapsFilter = capsFilter;
    videoCaptureBin->add(videoDevice);
    videoCaptureBin->add(capsFilter);
    
    QGstCapsPtr caps = QGstCaps::newEmpty();

    QGstStructure rgbStruct("video/x-raw-rgb");
    rgbStruct.setValue("width", frameSize.width());
    rgbStruct.setValue("height", frameSize.height());

    QGstStructure yuvStruct("video/x-raw-yuv");
    yuvStruct.setValue("width", frameSize.width());
    yuvStruct.setValue("height", frameSize.height());

    caps->makeWritable();
    caps->appendStructure(rgbStruct);
    caps->appendStructure(yuvStruct);

    videoCaptureBin->d->videoDeviceCapsFilter->setProperty("caps",caps);
    QGstElementPtr videoQueue = QGstElementFactory::make("queue");
    videoCaptureBin->d->videoQueue = videoQueue;
    videoCaptureBin->add(videoQueue);
    
    bool success = false;
    success = QGstElement::link(videoDevice,capsFilter,videoQueue);
    if (success) {
        QGstPadPtr pad = videoQueue->getStaticPad("src");
        QGstGhostPadPtr ghostPad = QGstGhostPad::newGhostPad("src",pad);

        videoCaptureBin->addPad(ghostPad);
        /*pad = videoCaptureBin->getStaticPad("src");
        gst_pad_add_buffer_probe(GST_PAD(pad->peekNativeObject()),G_CALLBACK(&bufferProbe),videoCaptureBin);*/
    }

    return VideoCaptureBinPtr(videoCaptureBin);
}

VideoCaptureBin::VideoCaptureBin()
    : QGstBin("video_capture_bin"), d(new VideoCaptureBinPrivate)
{
}

VideoCaptureBin::~VideoCaptureBin()
{
    delete d;
}

void VideoCaptureBin::newDevice(const QString &driver, const QString &devicePath)
{
    this->setState(QGstElement::Null);
    d->videoDevice->setState(QGstElement::Null);

    //remove old
    this->remove(d->videoDevice);

    //create new
    d->videoDevice = VideoInputDevice::newDevice(driver,devicePath);
    this->add(d->videoDevice);
}

void VideoCaptureBin::updateCapabilities(const QSize &targetSize,const QSize &frameRate)
{
    this->setState(QGstElement::Null);

    GstCaps *caps = gst_caps_new_full (
            gst_structure_new ("video/x-raw-rgb","width", G_TYPE_INT, targetSize.width(),
            "height", G_TYPE_INT, targetSize.height(),
            "framerate", GST_TYPE_FRACTION, frameRate.width(),frameRate.height(),
            NULL),
            gst_structure_new ("video/x-raw-yuv","width", G_TYPE_INT, targetSize.width(),
            "height", G_TYPE_INT, targetSize.height(),
            "framerate", GST_TYPE_FRACTION, frameRate.width(),frameRate.height(),
            NULL),
            NULL);
    if (caps) {
        kDebug() << "Updating video source capabilities";
        QGstCapsPtr capsPtr = QtGstreamer::QGstCaps::fromGstCaps(caps);
        d->videoDeviceCapsFilter->setProperty("caps",capsPtr);
        gst_caps_unref(caps);
        QGstElement::link(d->videoDevice,d->videoDeviceCapsFilter);
    }
}

void VideoCaptureBin::queryDeviceData()
{
    return d->videoDevice->queryDeviceData();
}

QList<QSize> VideoCaptureBin::getSupportedResolution() const
{
    return d->videoDevice->getSupportedResolution();
}

QList<QSize> VideoCaptureBin::getSupportedFrameRate(const QSize &resolution) const
{
    return d->videoDevice->getSupportedFrameRate(resolution);
}

void VideoCaptureBin::setBrightness(int value)
{
    if(d->videoDevice->hasColorBalance()) {
        d->videoDevice->setBrightness(value);
    }
}

int VideoCaptureBin::brightness()
{
    if(d->videoDevice->hasColorBalance()) {
        return d->videoDevice->brightness();
    }
    return 0;
}

void VideoCaptureBin::setContrast(int value)
{
    if(d->videoDevice->hasColorBalance()) {
        d->videoDevice->setContrast(value);
    }
}

int VideoCaptureBin::contrast()
{
    if(d->videoDevice->hasColorBalance()) {
        return d->videoDevice->contrast();
    }
    return 0;
}

void VideoCaptureBin::setSaturation(int value)
{
    if(d->videoDevice->hasColorBalance()) {
        d->videoDevice->setSaturation(value);
    }
}

int VideoCaptureBin::saturation()
{
    if(d->videoDevice->hasColorBalance()) {
        return d->videoDevice->saturation();
    }
    return 0;
}

void VideoCaptureBin::setHue(int value)
{
    if(d->videoDevice->hasColorBalance()) {
        d->videoDevice->setHue(value);
    }
}

int VideoCaptureBin::hue()
{
    if(d->videoDevice->hasColorBalance()) {
        return d->videoDevice->hue();
    }
}