/*
 * This file is part of the KDE project
 * Copyright (C) 2009 Eva Brucherseifer <eva.brucherseifer@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "killswitchdevice.h"
#include "killswitchmanager.h"

#include <QtCore/QDebug>
#include <QtDBus/QDBusConnection>
#include <QtDBus/QDBusInterface>
#include <QtDBus/QDBusReply>


class KillSwitchManagerPrivate
{
public:
    KillSwitchManagerPrivate() : manager("org.freedesktop.Hal",
                                   "/org/freedesktop/Hal/Manager",
                                   "org.freedesktop.Hal.Manager",
                                   QDBusConnection::systemBus()),
                          cacheSynced(false) { }

    QDBusInterface manager;
    QList<QString> devicesCache;
    bool cacheSynced;
};


KillSwitchManager::KillSwitchManager(QObject *parent)
    : QObject(parent),  d(new KillSwitchManagerPrivate())
{
    d->manager.connection().connect("org.freedesktop.Hal",
                                     "/org/freedesktop/Hal/Manager",
                                     "org.freedesktop.Hal.Manager",
                                     "DeviceAdded",
                                     this, SLOT(slotDeviceAdded(const QString &)));

    d->manager.connection().connect("org.freedesktop.Hal",
                                     "/org/freedesktop/Hal/Manager",
                                     "org.freedesktop.Hal.Manager",
                                     "DeviceRemoved",
                                     this, SLOT(slotDeviceRemoved(const QString &)));
}



KillSwitchManager::~KillSwitchManager()
{
   delete d;
}


QStringList KillSwitchManager::devicesFromQuery(const QString &parentUdi)
{
   if (!parentUdi.isEmpty())
   {
        QStringList result = findDeviceStringMatch("info.parent", parentUdi);

        const QStringList matches = result;
        result.clear();

        foreach (const QString &match, matches) {
             KillSwitchDevice device(match);

             if (device.queryDeviceInterface(QStringList("killswitch"))) {
                    result << match;
             }
        }
        return result;

    } else {
        return findDeviceByDeviceInterface(QStringList("killswitch"));
    }
}


QObject *KillSwitchManager::createDevice(const QString &udi)
{
    if (deviceExists(udi)) {
        return new KillSwitchDevice(udi);
    } else {
        return 0;
    }
}


bool KillSwitchManager::deviceExists(const QString &udi)
{
    if (d->devicesCache.contains(udi))
    {
        return true;
    }
    else if (d->cacheSynced)
    {
        return false;
    }

    QDBusReply<bool> reply = d->manager.call("DeviceExists", udi);

    if (!reply.isValid())
    {
        qWarning() << Q_FUNC_INFO << " error: " << reply.error().name() << endl;
        return false;
    }

    if (reply)
    {
        d->devicesCache.append(udi);
    }

    return reply;
}



QStringList KillSwitchManager::findDeviceStringMatch(const QString &key, const QString &value)
{
    QDBusReply<QStringList> reply = d->manager.call("FindDeviceStringMatch", key, value);

    if (!reply.isValid())
    {
        qWarning() << Q_FUNC_INFO << " error: " << reply.error().name() << endl;
        return QStringList();
    }

    return reply;
}

QStringList KillSwitchManager::findDeviceByDeviceInterface(const QStringList &cap_list)
{
    QStringList result;

    foreach (const QString &cap, cap_list)
    {
        QDBusReply<QStringList> reply = d->manager.call("FindDeviceByCapability", cap);

        if (!reply.isValid())
        {
            qWarning() << Q_FUNC_INFO << " error: " << reply.error().name() << endl;
            return QStringList();
        }
        if ( cap == QLatin1String( "video4linux" ) )
        {
            QStringList foundDevices ( reply );
            QStringList filtered;
            foreach ( const QString &udi, foundDevices )
            {
                QDBusInterface device( "org.freedesktop.Hal", udi, "org.freedesktop.Hal.Device", QDBusConnection::systemBus() );
                QDBusReply<QString> reply = device.call( "GetProperty", "video4linux.device" );
                if (!reply.isValid())
                {
                    qWarning() << Q_FUNC_INFO << " error getting video4linux.device: " << reply.error().name() << endl;
                    continue;
                }
                if ( !reply.value().contains( "video" ) )
                {
                    continue;
                }
                filtered.append( udi );
            }
            result += filtered;
        }
        else
        {
            result << reply;
        }
    }
    return result;
}

void KillSwitchManager::slotDeviceAdded(const QString &udi)
{
    KillSwitchDevice device(udi);
    if (device.queryDeviceInterface(QStringList("killswitch")))
    {
        d->devicesCache.append(udi);
        emit deviceAdded(udi);
    }
}

void KillSwitchManager::slotDeviceRemoved(const QString &udi)
{
    if (d->devicesCache.contains(udi))
    {
        d->devicesCache.removeAll(udi);
        emit deviceRemoved(udi);
    }
}


#include "killswitchmanager.moc"
