/***********************************************************************************
* Mini Player: Advanced media player for Plasma.
* Copyright (C) 2008 - 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef MINIPLAYERAPPLET_HEADER
#define MINIPLAYERAPPLET_HEADER

#include <QTimer>
#include <QKeyEvent>
#include <QDateTime>
#include <QVariantMap>
#include <QToolButton>
#include <QTextStream>
#include <QVBoxLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsLinearLayout>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsSceneDragDropEvent>

#include <Phonon/SeekSlider>
#include <Phonon/AudioOutput>
#include <Phonon/MediaObject>
#include <Phonon/MediaSource>
#include <Phonon/VideoWidget>
#include <Phonon/VolumeSlider>
#include <Phonon/MediaController>
#include <Phonon/ObjectDescription>
#include <Phonon/BackendCapabilities>

#include <KMenu>
#include <KAction>
#include <KDialog>
#include <KTabBar>
#include <KNotificationRestrictions>

#include <Solid/Device>
#include <Solid/OpticalDisc>
#include <Solid/StorageVolume>
#include <Solid/DeviceNotifier>
#include <Solid/DeviceInterface>
#include <Solid/PowerManagement>

#include <Plasma/Applet>
#include <Plasma/Dialog>
#include <Plasma/Slider>
#include <Plasma/ToolButton>

#include "ui_general.h"
#include "ui_video.h"
#include "ui_jumpToPosition.h"
#include "ui_playList.h"
#include "ui_fullScreen.h"

enum RepeatMode { NoRepeat = 0, RepeatTrack, RepeatPlayList };

class MiniPlayerPlayListModel;
class MiniPlayerPlayListImporter;

class MiniPlayerApplet : public Plasma::Applet
{
    Q_OBJECT

    public:
        MiniPlayerApplet(QObject *parent, const QVariantList &args);

        void init();
        void removeFromPlayList(KUrl::List items);
        void changeUrlTitle(KUrl url, QString title);
        void addToPlayList(KUrl::List items, bool play, int index = -1);
        void createPlayList(QString playList, KUrl::List tracks = KUrl::List());
        QVariantMap metaData(Phonon::MediaObject *mediaObject);
        KUrl::List currentPlayList();
        QList<QAction*> contextualActions();
        Phonon::MediaObject *mediaObject();
        Phonon::AudioOutput *audioOutput();
        KUrl currentUrl();
        QString deviceForUrl(KUrl url);
        QString titleForUrl(KUrl url);
        KIcon iconForUrl(KUrl url);
        KIcon iconForPlayList(QString playList);
        bool isPlayListRepeated();
        bool isTrackRepeated();
        bool isTrackRandom();
        bool eventFilter(QObject *object, QEvent *event);

    public slots:
        void configAccepted();
        void configReset();
        void stateChanged(Phonon::State newState, Phonon::State oldState);
        void hasVideoChanged(bool hasVideo);
        void volumeChanged(qreal volume);
        void mutedChanged(bool muted);
        void metaDataChanged();
        void ticked(qint64 position);
        void trackFinished();
        void trackPressed(QModelIndex index);
        void trackChanged(QModelIndex index);
        void moveUpTrack();
        void moveDownTrack();
        void playTrack(QModelIndex index = QModelIndex());
        void editTrackTitle();
        void copyTrackUrl();
        void removeTrack();
        void openFiles(bool playFirst = true);
        void openUrl();
        void openDisc(QAction *action);
        void play(KUrl url);
        void playPause();
        void playPrevious();
        void playNext();
        void play();
        void pause();
        void stop();
        void seekBackward();
        void seekForward();
        void increaseVolume();
        void decreaseVolume();
        void jumpToPosition();
        void toggleJumpToPosition();
        void toggleVolumeDialog();
        void togglePlayListDialog();
        void toggleFullScreen();
        void toggleMute();
        void toggleControls();
        void filterPlayList(QString text);
        void savePlayListNames();
        void movePlayList(int from, int to);
        void renamePlayList(int position);
        void removePlayList(int position);
        void visiblePlayListChanged(int position);
        void savePlayList();
        void addToPlayList();
        void exportPlayList();
        void newPlayList();
        void clearPlayList();
        void setActivePlayList(QString playList);
        void showVideoInDialog(bool show);
        void setPosition(int position);
        void setBrightness(int value);
        void setContrast(int value);
        void setHue(int value);
        void setSaturation(int value);
        void setRandomTrack(bool random);
        void setAspectRatio(int ratio);
        void setRepeatMode(int repeat);
        void changeAspectRatio(QAction *action);
        void changeAudioChannel(QAction *action);
        void changeSubtitles(QAction *action);
        void changeChapter(QAction *action);
        void changeAngle(QAction *action);
        void changeRepeatMode(QAction *action);
        void hideFullScreenControls();
        void showToolTip();
        void hideToolTip();
        void updateToolTip();
        void toolTipAboutToShow();
        void toolTipHidden();
        void savePlayListSettings(int position = 0, int index = 0);
        void importPlayList(QString playList, KUrl::List tracks, QHash<KUrl, QString> titles, bool play, int index);
        void addDevice(const QString &udi);
        void removeDevice(const QString &udi);
        void updateTheme();

    protected:
        void createConfigurationInterface(KConfigDialog *parent);
        void constraintsEvent(Plasma::Constraints);
        void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
        void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
        void dropEvent(QGraphicsSceneDragDropEvent *event);
        void keyPressEvent(QKeyEvent *event);
        void mousePressEvent(QGraphicsSceneMouseEvent *event);
        void toggleProxyWidget(bool show);
        void updateControls(Phonon::State state);
        QString timeToString(qint64 time);

    private:
        QHash<QString, MiniPlayerPlayListModel*> m_playLists;
        Phonon::VideoWidget *m_videoWidget;
        Phonon::AudioOutput *m_audioOutput;
        Phonon::MediaObject *m_mediaObject;
        Phonon::VolumeSlider *m_volumeSlider;
        Phonon::MediaController *m_mediaController;
        Plasma::Slider *m_seekSlider;
        Plasma::Dialog *m_volumeDialog;
        Plasma::Dialog *m_playListDialog;
        Plasma::ToolButton *m_volumeButton;
        QHash<QString, QAction*> m_discActions;
        QHash<QString, QString> m_discTitles;
        QHash<QString, QString> m_discPlayLists;
        QHash<QString, Phonon::DiscType> m_discTypes;
        QHash<KUrl, QString> m_titles;
        QList<QAction*> m_actions;
        QGraphicsWidget *m_controls;
        QGraphicsProxyWidget *m_proxyWidget;
        QGraphicsLinearLayout *m_mainLayout;
        QWidget *m_fullScreen;
        QDateTime m_lastClick;
        QTimer *m_hideFullScreenControls;
        QTimer *m_showPlayList;
        QTimer *m_hideToolTip;
        QString m_activePlayList;
        QString m_visiblePlayList;
        QString m_title;
        KDialog *m_jumpToPositionDialog;
        KAction *m_audioAction;
        KAction *m_playPauseAction;
        KAction *m_stopAction;
        KAction *m_navigationAction;
        KAction *m_fullScreenAction;
        KAction *m_controlsAction;
        KAction *m_playListAction;
        KAction *m_muteVolumeAction;
        KAction *m_videoAction;
        KMenu *m_openMenu;
        KMenu *m_aspectRatioMenu;
        KMenu *m_subtitleMenu;
        KMenu *m_chapterMenu;
        KMenu *m_angleMenu;
        KMenu *m_audioChannelMenu;
        KMenu *m_repeatModeMenu;
        KTabBar* m_playListTabBar;
        KNotificationRestrictions *m_notificationRestrictions;
        int m_stopSleepCookie;
        bool m_editorActive;
        Ui::general m_generalUi;
        Ui::video m_videoUi;
        Ui::jumpToPosition m_jumpToPositionUi;
        Ui::playList m_playListUi;
        Ui::fullScreen m_fullScreenUi;
};

#endif
