/***********************************************************************************
* Copyright (C) 2008 by Marco Martin <notmart@gmail.com>
* Copyright (C) 2009 by Michal Dutkiewicz <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#include "PlayerDBusHandler.h"
#include "PlayerAdaptor.h"
#include "MiniPlayerApplet.h"

#include <QDBusConnection>

QDBusArgument &operator << (QDBusArgument &argument, const DBusStatus &status)
{
    argument.beginStructure();
    argument << status.Play;
    argument << status.Random;
    argument << status.Repeat;
    argument << status.RepeatPlaylist;
    argument.endStructure();

    return argument;
}

const QDBusArgument &operator >> (const QDBusArgument &argument, DBusStatus &status)
{
    argument.beginStructure();
    argument >> status.Play;
    argument >> status.Random;
    argument >> status.Repeat;
    argument >> status.RepeatPlaylist;
    argument.endStructure();

    return argument;
}

PlayerDBusHandler::PlayerDBusHandler(MiniPlayerApplet *parent) : QObject(parent)
{
    m_player = parent;

    qDBusRegisterMetaType<DBusStatus>();

    setObjectName("PlayerDBusHandler");

    new PlayerAdaptor(this);

    QDBusConnection::sessionBus().registerObject("/Player", this);

    connect(m_player->mediaObject(), SIGNAL(stateChanged(Phonon::State, Phonon::State)), this, SLOT(stateChanged(Phonon::State, Phonon::State)));
    connect(m_player->mediaObject(), SIGNAL(seekableChanged(bool)), this, SLOT(seekableChanged(bool)));
    connect(m_player->mediaObject(), SIGNAL(metaDataChanged()), this, SLOT(trackChanged()));
}

void PlayerDBusHandler::PlayPause()
{
    m_player->playPause();
}

void PlayerDBusHandler::Play()
{
    m_player->play();
}

void PlayerDBusHandler::Pause()
{
    m_player->pause();
}

void PlayerDBusHandler::Stop()
{
    m_player->stop();
}

void PlayerDBusHandler::Next()
{
    m_player->playNext();
}

void PlayerDBusHandler::Prev()
{
    m_player->playPrevious();
}

void PlayerDBusHandler::Repeat(bool enable)
{
    m_player->setRepeatMode(enable?RepeatTrack:NoRepeat);
}

void PlayerDBusHandler::stateChanged(Phonon::State newState, Phonon::State oldState)
{
    Q_UNUSED(oldState)
    Q_UNUSED(newState)

    emit StatusChange(GetStatus());
    emit CapsChange(GetCaps());
}

void PlayerDBusHandler::seekableChanged(bool isSeekable)
{
    Q_UNUSED(isSeekable)

    emit CapsChange(GetCaps());
}

void PlayerDBusHandler::trackChanged()
{
    emit TrackChange(GetMetadata());
}

void PlayerDBusHandler::VolumeSet(int value)
{
    m_player->audioOutput()->setVolume(qreal(value) / 100.0);
}

void PlayerDBusHandler::PositionSet(int position)
{
    m_player->setPosition(position);
}

DBusStatus PlayerDBusHandler::GetStatus()
{
    DBusStatus status;

    switch (m_player->mediaObject()->state())
    {
        case Phonon::PlayingState:
            status.Play = 0;
        break;
        case Phonon::PausedState:
            status.Play = 1;
        break;
        case Phonon::StoppedState:
        default:
            status.Play = 2;
    }

    status.Random = (m_player->isTrackRandom()?1:0);
    status.Repeat = (m_player->isTrackRepeated()?1:0);
    status.RepeatPlaylist = (m_player->isPlayListRepeated()?1:0);

    return status;
}

QVariantMap PlayerDBusHandler::GetMetadata()
{
    return m_player->metaData(m_player->mediaObject());
}

int PlayerDBusHandler::GetCaps()
{
    int caps = NONE;

    caps |= CAN_GO_NEXT;

    caps |= CAN_GO_PREV;

    if (m_player->mediaObject()->state() == Phonon::PlayingState)
    {
        caps |= CAN_PAUSE;
    }

    if (m_player->mediaObject()->state() == Phonon::PausedState)
    {
        caps |= CAN_PLAY;
    }

    if (m_player->mediaObject()->isSeekable())
    {
        caps |= CAN_SEEK;
    }

    caps |= CAN_PROVIDE_METADATA;

    caps |= CAN_HAS_TRACKLIST;

    return caps;
}

int PlayerDBusHandler::VolumeGet() const
{
     return (m_player->audioOutput()->volume() * 100);
}

qint64 PlayerDBusHandler::PositionGet() const
{
    return m_player->mediaObject()->currentTime();
}

#include "PlayerDBusHandler.moc"
