/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "gamelist.h"
#include "game.h"
#include "gamewidget.h"
#include "hoverwidget.h"
#include "animatedlayout.h"

// Qt
#include <QGraphicsLinearLayout>
#include <QDir>
#include <QGraphicsSceneDragDropEvent>

// KDE
#include <KDebug>



GameList::GameList(QGraphicsWidget *parent)
    : QGraphicsWidget(parent)
{

    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    
    m_layout = new AnimatedLayout(Qt::Vertical);
    setLayout(m_layout);

    m_hoverWidget = new HoverWidget(this);
    m_hoverWidget->setVisible(false);
       
    syncBorders();
        
    setAcceptHoverEvents(true);
    setAcceptDrops(true);
    
    m_moveIndex = -1;
 
    m_sort = GameList::NoSort;
    
}


GameList::~GameList()
{

    delete m_hoverWidget;
    
}


void GameList::syncBorders()
{
    
    m_layout->setSpacing(m_hoverWidget->spacing());
    qreal top, left, right, bottom;
    m_hoverWidget->getMargins(left, top, right, bottom);
    setContentsMargins(top, left, right, bottom);
    
}


void GameList::widgetHover(const QPointF &pos)
{

    if (pos == QPointF(-1, -1)) {
        return;
    }
    
    QGraphicsWidget *target = static_cast<QGraphicsWidget*>(sender());
    m_hoverWidget->setTarget(target);

}


void GameList::appendGame(Game *game)
{

    insertGame(m_layout->count(), game);
    
}


void GameList::insertGame(const int &index, Game *game)
{

    GameWidget *widget = new GameWidget(this, game);
    connect(widget, SIGNAL(hover(QPointF)), this, SLOT(widgetHover(QPointF)));
    connect(widget, SIGNAL(up()), this, SLOT(upClicked()));
    connect(widget, SIGNAL(down()), this, SLOT(downClicked()));
    connect(widget, SIGNAL(edit()), this, SLOT(editClicked()));
    connect(widget, SIGNAL(remove()), this, SLOT(removeClicked()));
    connect(widget->game(), SIGNAL(changed()), this, SLOT(gameChanged()));
    widget->setSortingEnabled((sort() == GameList::NoSort));
        
    if (sort() == GameList::NoSort) {
        m_layout->insertItem(index, widget);
    } else {
        QList<Game*> newList = games();
        newList.append(game);
        qStableSort(newList.begin(), newList.end(), Game::lessThan);
        m_layout->insertItem(newList.indexOf(game), widget);
    }

}


void GameList::move(const QPointF &target, const QPointF &destination)
{

    QGraphicsWidget *targetWidget = widgetAt(target);
    QGraphicsWidget *destinationWidget = widgetAt(destination); 
        
    int from = indexOf(targetWidget);
    int to = indexOf(destinationWidget);
    
    move(from, to);
    
}


void GameList::move(const int &from, const int &to)
{

    if (sort() != GameList::NoSort || from == -1) {
        return;
    }
    
    GameWidget *widget = static_cast<GameWidget*>(m_layout->itemAt(from));
        
    m_layout->removeAt(from);
    m_layout->insertItem(to, widget);
    
    m_hoverWidget->setTarget(0);
    widget->hideButtons();
    widget->game()->moved();

}


void GameList::setSortOrder(const GameList::SortOrder &sort)
{

    if (m_sort == sort) {
        return;
    }
        
    for (int i = 0; i < m_layout->count(); i++) {
        GameWidget *widget = static_cast<GameWidget*>(m_layout->itemAt(i)); 
        widget->setSortingEnabled((sort == GameList::NoSort));
    }
    m_sort = sort;

    gameChanged();
    
}


void GameList::remove(QGraphicsWidget *widget)
{

    const int index = indexOf(widget);
    if (index == -1) {
        return;
    }
    GameWidget *gWidget = static_cast<GameWidget*>(widget);
    m_layout->removeItem(gWidget);
    gWidget->game()->remove();
    gWidget->deleteLater();

    if (isEmpty()) {
        m_hoverWidget->hide();
    }
    
}


void GameList::mousePressEvent(QGraphicsSceneMouseEvent *event)
{

    event->ignore();
    
}


void GameList::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{

    event->ignore();

}


void GameList::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
{

    event->ignore();
    
}


void GameList::keyPressEvent(QKeyEvent *event)
{

    event->ignore();

}


void GameList::wheelEvent(QGraphicsSceneWheelEvent *event)
{
    
    event->ignore();
    
}


void GameList::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{

    event->ignore();
    
}


void GameList::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{

    Q_UNUSED(event);
    m_hoverWidget->setTarget(0);
   
}


void GameList::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
{
    
    
    QList<KUrl> urls = KUrl::List::fromMimeData(event->mimeData());
    if (!event->mimeData()->data("Pos").isEmpty() || !urls.isEmpty()) {
        event->accept();
    } else {
        event->ignore();
    }

}


void GameList::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
{

    Q_UNUSED(event);

}


void GameList::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
{

    Q_UNUSED(event);

}


void GameList::dropEvent(QGraphicsSceneDragDropEvent *event)
{

    kDebug() << "drop";
    QList<KUrl> urls = KUrl::List::fromMimeData(event->mimeData());
    // internal                                                       
    if (!event->mimeData()->data("Pos").isEmpty()) {                                                                
        const QString str = event->mimeData()->data("Pos");
        QPointF pos;
        
        kDebug() << "pos" << str;
        
        const QString x = str.section(' ', 0);
        const QString y = str.section(' ', 1);
        
        pos.setX(x.toInt());
        pos.setY(y.toInt());
        
        move(pos, event->pos());
        
        event->accept();                                                  
    } else if(urls.count() >= 1) {
        KUrl file = urls.first();

        if (!file.isLocalFile()) {
            KService::Ptr service = KService::serviceByStorageId(file.path());
            if (service && service->isValid()) {                                                                   
                QString path = service->entryPath();  
                if (!path.isEmpty() && QDir::isAbsolutePath(path)) {
                    emit gameDrop(event->pos(), service->entryPath());
                    event->accept();
                    return;
                } else {
                    kDebug() << "Invalid drop:" << file << path;
                }                         
            } else {
                kDebug() << "Entry not found:" << file;
            }
            event->ignore();
            return;
        }
        emit gameDrop(event->pos(), file);
        event->accept();
    } else {
        kDebug() << "no url, no pos...";
        event->ignore();
    }
    
}
    

QGraphicsWidget *GameList::widgetAt(const QPointF &pos)
{

    if (m_layout->count() == 0) {
        return 0;
    }
    
    QGraphicsWidget *last = 0;
    for (int i = 0; i < m_layout->count(); i++) {
        if (m_layout->itemAt(i)->geometry().contains(pos)) {
            return static_cast<QGraphicsWidget*>(m_layout->itemAt(i));
        } else if (pos.y() < m_layout->itemAt(i)->geometry().topLeft().y()) {
            if (last) {
                return last;
            } else {
                return static_cast<QGraphicsWidget*>(m_layout->itemAt(i));
            }
        }
        last = static_cast<QGraphicsWidget*>(m_layout->itemAt(i));
    }
    return 0;

}


int GameList::indexOf(QGraphicsWidget *widget)
{
    
    for (int i = 0; i < m_layout->count(); i++) {
        if (m_layout->itemAt(i) == widget) {
            return i;
        } 
    } 
    return -1;
    
}


QList<Game*> GameList::games() const
{

    QList<Game*> list;
    for (int i = 0; i < m_layout->count(); i++) {
        GameWidget *widget = static_cast<GameWidget*>(m_layout->itemAt(i));
        list.append(widget->game());
    }
    return list;
    
}


bool GameList::isEmpty() const
{

    return (m_layout->count() == 0);

}


GameList::SortOrder GameList::sort() const
{

    return m_sort;

}


void GameList::upClicked()
{

    QGraphicsWidget *widget = static_cast<QGraphicsWidget*>(sender());
    const int from = indexOf(widget);
    const int to = from-1;
    
    if (to < 0 || to >= m_layout->count()) {
        return;
    }
    move(from, to);
    
}


void GameList::downClicked()
{

    QGraphicsWidget *widget = static_cast<QGraphicsWidget*>(sender());
    const int from = indexOf(widget);
    const int to = from+1;
    
    if (to < 0 || to >= m_layout->count()) {
        return;
    }
    move(from, to);
    
}


void GameList::removeClicked()
{

    remove(static_cast<QGraphicsWidget*>(sender()));

}


void GameList::editClicked()
{

    GameWidget *widget = static_cast<GameWidget*>(sender());
    widget->game()->edit();
    
}


void GameList::gameChanged()
{

    if (m_sort == GameList::NoSort) {
        return;
    }

    const QList<Game*> currentList = games();
    QList<Game*> newList = currentList;
    
    qStableSort(newList.begin(), newList.end(), Game::lessThan);
    
    if (newList != currentList) {
        QList<GameWidget*> widgets;
        while (m_layout->count() > 0) {
            widgets.append(static_cast<GameWidget*>(m_layout->itemAt(0)));
            m_layout->removeAt(0);
        }
        
        
        for (int i = 0; i < newList.size(); i++) {
            foreach (GameWidget *widget, widgets) {
                if (widget->game() == newList[i]) {
                    m_layout->addItem(widget);
                    break;
                }
            }
        }
    }
    
}


#include "gamelist.moc"

