/*
 *   Copyright (C) 2008 Teemu Rytilahti <tpr@d5k.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 (or, at
 *   your option, any later version) as published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


#include "mythengine.h"

#include <KUrl>
#include <KDebug>

#include <QByteArray>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>


MythTVEngine::MythTVEngine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent, args)
{
}

MythTVEngine::~MythTVEngine()
{
}

bool MythTVEngine::sourceRequestEvent(const QString &name)
{
    return updateSourceEvent(name);
}
 
bool MythTVEngine::updateSourceEvent(const QString &name)
{
    m_url = KUrl("http://" + name + "/xml");
    m_data.clear();
    m_transferJob = KIO::get(m_url, KIO::NoReload, KIO::HideProgressInfo);
    connect(m_transferJob, SIGNAL(data(KIO::Job *, const QByteArray &)), this, SLOT(dataFetched(KIO::Job *, const QByteArray &)));
    connect(m_transferJob, SIGNAL(result(KJob *)), this, SLOT(fetchingDone(KJob *)));

    return true;
}

void MythTVEngine::dataFetched(KIO::Job *job, const QByteArray &data)
{
    Q_UNUSED(job);
    m_data.append(data);
    //updateData();
}


void MythTVEngine::fetchingDone(KJob *job)
{
    Q_UNUSED(job);
    QDomDocument doc("data");
    doc.setContent(m_data);
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while(!n.isNull()) {
        if(n.isElement()) {
            QDomElement e = n.toElement();
            if(e.tagName() == "Encoders") {
                parseEncoders(n);
                //break;
            }
            if(e.tagName() == "Scheduled") {
                parseScheduled(n);
                //break;
            }
            if(e.tagName() == "JobQueue") {
                //parseQueue(n);
                //break;
            }
            if(e.tagName() == "MachineInfo") {
                parseMachineInfo(n);
                
            }
        }
        n = n.nextSibling();
     }
}

void MythTVEngine::parseEncoders(const QDomNode &node)
{
    QDomNode n(node.firstChild());
    int count = node.toElement().attribute("count").toInt();
    if(count == 0)
        return;
    
    for(int i=1;i<=count;i++) {
        if(n.isElement()) {
            QDomElement e = n.toElement();
            setData(QString("Encoder %1").arg(i), "Local", e.attribute("local"));
            setData(QString("Encoder %1").arg(i), "Connected", e.attribute("connected"));
            setData(QString("Encoder %1").arg(i), "Id", e.attribute("id"));
            setData(QString("Encoder %1").arg(i), "State", e.attribute("state"));
            setData(QString("Encoder %1").arg(i), "Hostname", e.attribute("hostname"));
            break;
        }
        n = n.nextSibling();
    }
}

void MythTVEngine::parseScheduled(const QDomNode &node)
{
    QDomNode n(node.firstChild());
    int count = node.toElement().attribute("count").toInt();
    if(count == 0)
        return;
    
    for(int i=1;i<=count;i++) {
        if(n.isElement()) {
            QDomElement e = n.toElement();
            setData(QString("Scheduled %1").arg(i), "Flags", e.attribute("programFlags")); // what is this?
            setData(QString("Scheduled %1").arg(i), "Title", e.attribute("title"));
            setData(QString("Scheduled %1").arg(i), "Id", e.attribute("programId"));
            setData(QString("Scheduled %1").arg(i), "Category Type", e.attribute("catType"));
            setData(QString("Scheduled %1").arg(i), "Category", e.attribute("category"));
            setData(QString("Scheduled %1").arg(i), "Series Id", e.attribute("seriesId"));
            setData(QString("Scheduled %1").arg(i), "Endtime", e.attribute("endTime"));
            setData(QString("Scheduled %1").arg(i), "Last Modified", e.attribute("lastModified"));
            setData(QString("Scheduled %1").arg(i), "Subtitle", e.attribute("subTitle"));
            setData(QString("Scheduled %1").arg(i), "Stars", e.attribute("stars"));
            setData(QString("Scheduled %1").arg(i), "Repeat", e.attribute("repeat"));
            setData(QString("Scheduled %1").arg(i), "Filesize", e.attribute("fileSize"));
            setData(QString("Scheduled %1").arg(i), "Starttime", e.attribute("startTime"));
            setData(QString("Scheduled %1").arg(i), "Hostname", e.attribute("hostname"));
            setData(QString("Scheduled %1").arg(i), "Description", e.text());
        }
        n = n.nextSibling();
    }
}

void MythTVEngine::parseMachineInfo(const QDomNode &node)
{
    QDomNode n(node.firstChild());
    while(!n.isNull()) {
        if(n.isElement()) {
            QDomElement e = n.toElement();
            if(e.tagName() == "Storage") {
                setData("Drive Total", "Free", e.attribute("drive_total_free"));
                setData("Drive Total", "Used", e.attribute("drive_total_used"));
                setData("Drive Total", "Total", e.attribute("drive_total_total"));
                setData("Drive Total", "Dirs", e.attribute("drive_total_dirs"));
                
                // TODO: check for other drives also
                setData("Drive 1", "Free", e.attribute("drive_1_free"));
                setData("Drive 1", "Used", e.attribute("drive_1_used"));
                setData("Drive 1", "Total", e.attribute("drive_1_total"));
                setData("Drive 1", "Dirs", e.attribute("drive_1_dirs"));
                
                setData("Drive", "Drive fsID", e.attribute("fsids"));
            }
            if(e.tagName() == "Load") {
                QString load = QString("%1 %2 %3").arg(e.attribute("avg1")).arg(e.attribute("avg2")).arg(e.attribute("avg3"));
                setData("General", "Load", load);  
            }
            if(e.tagName() == "Guide") {
                setData("Guide", "Days", e.attribute("guideDays"));
                setData("Guide", "Status", e.attribute("status"));
                setData("Guide", "Next Update", e.attribute("next"));
                setData("Guide", "Guide Ends", e.attribute("end"));
                setData("Guide", "Guide Thru", e.attribute("guideThru"));
                setData("Guide", "Guide Starts", e.attribute("start"));
            }
        }
        n = n.nextSibling();
    }
    
}
#include "mythengine.moc"
