/***************************************************************************
 *   Copyright (C) 2009 by Shawn Starr <shawn.starr@rogers.com>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Ion for BuienRadar.NL XML data */

#ifndef _ION_BUIENRADAR_H
#define _ION_BUIENRADAR_H

#include <QtXml/QXmlStreamReader>
#include <QtCore/QStringList>
#include <QDebug>
#include <QDateTime>
#include <kurl.h>
#include <kio/job.h>
#include <kio/scheduler.h>
#include <kdemacros.h>
#include <plasma/dataengine.h>
#include <plasma/weather/ion.h>
#include <plasma/weather/weatherutils.h>

class WeatherData
{

public:
    QString locationName; // stationnaam
    QString stationCode;
    QString stationLat; 
    QString stationLon;

    // Current observation information.
    QString observationTime; 	// datum
    QString weather; 		// icoonactueel

    QString temperature_C; 	// temperatuur
    QString humidity;		// luchtvochtigheid
    QString windDirection; 	// windrichting
    QString windSpeed_MS;	// windsnelheidMS - Float value
    QString windGust; 		// windstoten  - Float value
    QString pressure; 		// luchtdruk
    QString visibility; 	// zichtmeters

    // Five day forecast
    struct ForecastInfo {
        QString forecastPeriod; // dagweek
        QString forecastSummary; // icon name == forecast in this case
        QString iconName; // icoon

        QString forecastTempHigh; // maxtemp
        QString forecastTempLow; // mintemp
        QString popPrecent; // kansregen (chance of rain)
        QString windForecast; // windkracht (wind speed)
        QString windDirForecast; // windrichting (wind direction)
    };

};

class KDE_EXPORT BuienradarIon : public IonInterface
{
    Q_OBJECT

public:
    BuienradarIon(QObject *parent, const QVariantList &args);
    ~BuienradarIon();
    void init(void);  // Setup the city location, fetching the correct URL name.
    bool updateIonSource(const QString& source); // Sync data source with Applet
    void updateWeather(const QString& source);

public Q_SLOTS:
    virtual void reset(void);

protected Q_SLOTS:
    void setup_slotDataArrived(KIO::Job *, const QByteArray &);
    void setup_slotJobFinished(KJob *);

    void slotDataArrived(KIO::Job *, const QByteArray &);
    void slotJobFinished(KJob *);

private:
    /* Buienradar Methods - Internal for Ion */
    QMap<QString, ConditionIcons> setupDayIconMappings(void) const;
    QMap<QString, ConditionIcons> setupNightIconMappings(void) const;
    QMap<QString, ConditionIcons> setupForecastIconMappings(void) const;
   
    QMap<QString, ConditionIcons> const& nightIcons(void) const;
    QMap<QString, ConditionIcons> const& dayIcons(void) const;
    QMap<QString, ConditionIcons> const& forecastIcons(void) const;
 
    QMap<QString, WindDirections> setupWindIconMappings(void) const;
    QMap<QString, WindDirections> const& windIcons(void) const;

    // Place information
    QString const country(const QString& source) const;
    QString place(const QString& source) const;
    QString station(const QString& source) const;
    QString latitude(const QString& source) const;
    QString longitude(const QString& source) const;

    // Current Conditions Weather info
    QString observationTime(const QString& source) const;
    int periodHour(const QString& source) const;
    QString condition(const QString& source) const;
    QMap<QString, QString> temperature(const QString& source) const;
    QString dewpoint(const QString& source) const;
    QMap<QString, QString> humidity(const QString& source) const;
    QMap<QString, QString> visibility(const QString& source) const;
    QMap<QString, QString> pressure(const QString& source) const;
    QMap<QString, QString> wind(const QString& source) const;

    // Load and Parse the place XML listing
    void getXML(void) const;
    bool readXML(void);

    // Load and parse the specific place(s)
    void getXMLData(const QString& source);
    bool readXMLData(const QString& source, QXmlStreamReader& xml);

    // Check if place specified is valid or not
    QStringList validate(const QString& source) const;

    // Catchall for unknown XML tags
    void parseUnknownElement(QXmlStreamReader& xml) const;

    // Parse weather XML data
    void parseStation(WeatherData& data, QXmlStreamReader& xml);
    void parseForecast(WeatherData& data, QXmlStreamReader& xml);
    void parseDayForecast(WeatherData& data, QXmlStreamReader& xml);
    void parseWeatherStations(WeatherData& data, QXmlStreamReader& xml);
    void parseStationList(WeatherData& data, QXmlStreamReader& xml);

private:
    class Private;
    Private * d;
};

K_EXPORT_PLASMA_DATAENGINE(buienradar, BuienradarIon)

#endif
