/*
    This file is part of the KDE project.
    Copyright (c) 2006 Friedrich W. H. Kossebau <Friedrich.W.H@Kossebau.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301  USA
*/



// qt specific
#include <qpopupmenu.h>
// kde specific
#include <kpopupmenu.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kapplication.h>
// menu specific
#include "lazyfillmenu.h"
#include "kuserface.h"
#include "klocalsessionmanager.h"
#include "ksessionmenufiller.h"



static const int LockOnSwitchId = 50;
static const int NewSessionId = 0;


KSessionMenuFiller::KSessionMenuFiller( const KUser U, KLocalSessionManager *SM )
: User( U ),
  SessionManager( SM )
{
}


int KSessionMenuFiller::fill( QPopupMenu *Menu )
{
  int EntriesBefore = Menu->count();
  if( SessionManager->canSwitchSession() )
  {
    // other sessions
    KSessionList Sessions = SessionManager->localSessions();
    if( Sessions.size()>0 )
    {
      for( KSessionList::ConstIterator it = Sessions.begin(); it != Sessions.end(); ++it )
        insertSessionItem( Menu, *it );

      // anything added?
      if( Menu->count()>0 )
        Menu->insertSeparator();
    }

    // new session
    int p;
    if( SessionManager->canCreateSession() && (p = SessionManager->numberOfReservedSessions()) >= 0 )
    {
      Menu->insertItem( SmallIconSet("fork"), i18n("Log In..."), this, SLOT(activateSession(int)), 0, NewSessionId ); // KDE 4 icon name: probably system-switch-user (?), or maybe go-jump... but there is no real "log in" icon
      if( p == 0 )
        Menu->setItemEnabled( NewSessionId, false );
      Menu->insertSeparator();
    }

    {
      // add section title
      KPopupTitle *Title = new KPopupTitle( Menu );
      Title->setTitle(i18n("Other Sessions"));
      Menu->insertItem( Title, -1, 0 );

      if( SessionManager->canLockSession() )
      {
        // add toggle for locking
        Menu->insertItem( i18n("Lock Current Session on Switch"), this, SLOT(toggleLockOnSwitch()), 0, LockOnSwitchId );
        Menu->setItemChecked( LockOnSwitchId, SessionManager->doesLockOnSwitch() );
      }
    }
  }

  unsigned int CountBefore = Menu->count();

  // save session TODO: does not exist in KRootWM!!!!
  if( SessionManager->canSaveSession() )
     Menu->insertItem( SmallIconSet("filesave"), i18n("Save Session"), SessionManager, SLOT(saveSession()) );
  // lock session
  if( SessionManager->canLockSession() )
      Menu->insertItem( SmallIconSet("lock"), i18n("Lock Session"), SessionManager, SLOT(lockSession()) );
  // end session
  if( SessionManager->canLogoutSession() )
      Menu->insertItem( SmallIconSet("exit"), i18n("Log Out..."), SessionManager, SLOT(logoutSession()) );

  // anything added?
  if( Menu->count()>CountBefore )
  {
    // add section title
    KPopupTitle *Title = new KPopupTitle(Menu);
    Title->setTitle( i18n("Current Session") );
    Menu->insertItem( Title,-1,CountBefore );
  }

  return Menu->count() - EntriesBefore;
}


/*
Different listings:
a) simple one:
only other graphical sessions, show as users
b)

TODO: make lock when switching a check item, save setting globally
*/

static QString renderSessionDescription( KUser User, const KSession &Session, const QString &Pattern )
{
  QString Description;
  int l = Pattern.length();
  for( int i = 0; i < l; ++i )
  {
    if (Pattern[i] == '%')
    {
      switch (Pattern[++i].latin1())
      {
        //case 'U': Description += User.displayedName(); break;
        case 'F': Description += User.fullName(); break;
        case 'L': Description += User.loginName(); break;
        case 'S': Description += Session.Session; break;
        case 'D': Description += Session.DisplayName; break;
        case 'V': Description += Session.VTId; break;
        case '%': ;
        default:  Description += Pattern[i];
      }
    }
    else
      Description += Pattern[i];
  }
  return Description;
}


//static bool ListPlainTerminals = false;
static bool ListCurrent = false;
static const QString DefaultLocalPattern("%F");

void KSessionMenuFiller::insertSessionItem( QPopupMenu *Menu, const KSession &Session )
{
  // ignore ttys and current session
  if( Session.IsTty )
    return;
  if( !ListCurrent && Session.Current )
    return;

  int VTId = Session.VTId;
  QString Text;
  QIconSet IconSet;
  KUser User( Session.LoginName );
  if( User.isValid() )
  {
    Text = renderSessionDescription( User, Session, DefaultLocalPattern );
//     Text = i18n("fullname (loginname): session type", "%1 (%2): %3").arg( User.fullName(), User.loginName(), Session.Session );

//     QString Location = VTId ? QString("%1, vt%2").arg( Session.DisplayName ).arg( VTId ) :
//                     Session.DisplayName;
//     QString Text = i18n( "%1: Terminal on %2" ).arg( UserName, Location );

    IconSet = KUserFaceLoader::self()->loadFaceIconSet( User, KIcon::Small );
  }
  // unknown user
  else
  {
    // TODO: enable pattern here, too
    Text = Session.Session.isEmpty() ?     i18n("Unused") :
                   Session.Session == "<remote>" ? i18n("X login on remote host") : // TODO: which?
                   i18n("X login on %1").arg( Session.Session );
    //QString Location = VTId ? QString("%1, vt%2").arg( Session.DisplayName ).arg( VTId ) :
    //                   Session.DisplayName;
    //QString Text = i18n( "%1: Terminal on %2" ).arg( UserName, Location );

    IconSet = SmallIconSet( "x" );
  }

  int Id = Menu->insertItem( IconSet, Text, this, SLOT(activateSession(int)), 0, VTId+1 );

  // TODO: in which cases can vt be 0?
  if( VTId == 0 )
    Menu->setItemEnabled( Id, false );
}



void KSessionMenuFiller::toggleLockOnSwitch()
{
  bool NewState = !SessionManager->doesLockOnSwitch();
  SessionManager->setLockOnSwitch( NewState );
}


void KSessionMenuFiller::activateSession( int EntryId )
{
  if( EntryId == NewSessionId )
    SessionManager->createSession();
  else //if( EntryId>=BaseSessionId /*&& !isItemChecked(EntryId)*/ )
    SessionManager->switchToSession( EntryId-1 );
}

#include "ksessionmenufiller.moc"
